#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# States
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.states.pending.title",
    ca = u"Pendent d'avaluar",
    es = u"Pendiente de evaluar",
    en = u"Evaluation pending"
)

translations.define(
    "woost.extensions.translationworkflow.states.pending.plural_title",
    ca = u"Pendents d'avaluar",
    es = u"Pendientes de evaluar",
    en = u"Evaluation pending"
)

translations.define(
    "woost.extensions.translationworkflow.states.ignored.title",
    ca = u"Ignorada",
    es = u"Ignorada",
    en = u"Ignored"
)

translations.define(
    "woost.extensions.translationworkflow.states.ignored.plural_title",
    ca = u"Ignorades",
    es = u"Ignoradas",
    en = u"Ignored"
)

translations.define(
    "woost.extensions.translationworkflow.states.silenced.title",
    ca = u"Silenciada",
    es = u"Silenciada",
    en = u"Silenced"
)

translations.define(
    "woost.extensions.translationworkflow.states.silenced.plural_title",
    ca = u"Silenciades",
    es = u"Silenciadas",
    en = u"Silenced"
)

translations.define(
    "woost.extensions.translationworkflow.states.selected.title",
    ca = u"Seleccionada",
    es = u"Seleccionada",
    en = u"Selected"
)

translations.define(
    "woost.extensions.translationworkflow.states.selected.plural_title",
    ca = u"Seleccionades",
    es = u"Seleccionadas",
    en = u"Selected"
)

translations.define(
    "woost.extensions.translationworkflow.states.sent_to_agency.title",
    ca = u"Enviada a agència",
    es = u"Enviada a agencia",
    en = u"Sent to agency"
)

translations.define(
    "woost.extensions.translationworkflow.states.sent_to_agency.plural_title",
    ca = u"Enviades a agència",
    es = u"Enviadas a agencia",
    en = u"Sent to agency"
)

translations.define(
    "woost.extensions.translationworkflow.states.in_translation.title",
    ca = u"En curs",
    es = u"En curso",
    en = u"In translation"
)

translations.define(
    "woost.extensions.translationworkflow.states.in_translation.plural_title",
    ca = u"En curs",
    es = u"En curso",
    en = u"In translation"
)

translations.define(
    "woost.extensions.translationworkflow.states.proposed.title",
    ca = u"Proposada",
    es = u"Propuesta",
    en = u"Proposed"
)

translations.define(
    "woost.extensions.translationworkflow.states.proposed.plural_title",
    ca = u"Proposades",
    es = u"Propuestas",
    en = u"Proposed"
)

translations.define(
    "woost.extensions.translationworkflow.states.applied.title",
    ca = u"Aplicada",
    es = u"Aplicada",
    en = u"Applied"
)

translations.define(
    "woost.extensions.translationworkflow.states.applied.plural_title",
    ca = u"Aplicades",
    es = u"Aplicadas",
    en = u"Applied"
)

# Transitions
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.transitions.ignore.title",
    ca = u"Ignorar",
    es = u"Ignorar",
    en = u"Ignore"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.silence.title",
    ca = u"Silenciar",
    es = u"Silenciar",
    en = u"Silence"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.select.title",
    ca = u"Seleccionar",
    es = u"Seleccionar",
    en = u"Select"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.reject_selection.title",
    ca = u"Rebutjar",
    es = u"Rechazar",
    en = u"Reject"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.assign.title",
    ca = u"Assignar",
    es = u"Asignar",
    en = u"Assign"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.propose.title",
    ca = u"Proposar traducció",
    es = u"Proponer traducción",
    en = u"Propose translation"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.cancel_proposal.title",
    ca = u"Anul·lar proposta",
    es = u"Anular propuesta",
    en = u"Cancel proposal"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.reject.title",
    ca = u"Rebutjar",
    es = u"Rechazar",
    en = u"Reject"
)

translations.define(
    "woost.extensions.translationworkflow.transitions.accept.title",
    ca = u"Acceptar",
    es = u"Aceptar",
    en = u"Accept"
)

# Transition icons
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.icons.ignore.title",
    ca = u"Icona per la transició 'Ignorar' del circuït de traduccions",
    es = u"Icono para la transición 'Ignorar' del circuito de traducciones",
    en = u"Icon for the 'Ignore' transition of the translation workflow"
)

translations.define(
    "woost.extensions.translationworkflow.icons.silence.title",
    ca = u"Icona per la transició 'Silenciar' del circuït de traduccions",
    es = u"Icono para la transición 'Silenciar' del circuito de traducciones",
    en = u"Icon for the 'Silence' transition of the translation workflow"
)

translations.define(
    "woost.extensions.translationworkflow.icons.forward.title",
    ca = u"Icona per les transicions positives del circuït de traduccions",
    es = u"Icono para las transiciones positivas del circuito de traducciones",
    en = u"Icon for the positive transitions of the translation workflow"
)

translations.define(
    "woost.extensions.translationworkflow.icons.backward.title",
    ca = u"Icona per les transicions negatives del circuït de traduccions",
    es = u"Icono para las transiciones negativas del circuito de traducciones",
    en = u"Icon for the negative transitions of the translation workflow"
)

# Roles
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.roles.coordinators.title",
    ca = u"Coordinadors de traducció",
    es = u"Coordinadores de traducción",
    en = u"Translation coordinators"
)

translations.define(
    "woost.extensions.translationworkflow.roles.translators.title",
    ca = u"Traductors",
    es = u"Traductores",
    en = u"Translators"
)

# Permissions
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow."
    "permissions.translators.modify_assigned_requests.subject_description",
    ca = u"peticions assignades al traductor en estat de traducció",
    es = u"peticiones asignadas al traductor en estado de traducción",
    en = u"requests assigned to the translator and in translation"
)

translations.define(
    "woost.extensions.translationworkflow."
    "permissions.translators.read_requests.subject_description",
    ca = u"peticions assignades al traductor",
    es = u"peticiones asignadas al traductor",
    en = u"requests assigned to the translator"
)

translations.define(
    "woost.extensions.translationworkflow."
    "permissions.translators.read_comments.subject_description",
    ca = u"comentaris de peticions assignades al traductor",
    es = u"comentarios de peticiones asignadas al traductor",
    en = u"comments for requests assigned to the translator"
)

translations.define(
    "woost.extensions.translationworkflow."
    "permissions.translators.write_comments.subject_description",
    ca = u"comentaris de peticions assignades al traductor",
    es = u"comentarios de peticiones asignadas al traductor",
    en = u"comments for requests assigned to the translator"
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2009
"""
from __future__ import with_statement
from cocktail.translations import set_language
from woost.tests.models.basetestcase import BaseTestCase


class IsAccessibleExpressionTestCase(BaseTestCase):

    def setUp(self):
        from woost.models import User
        BaseTestCase.setUp(self)
        set_language("en")
        self.user = User()
        self.user.insert()

    def accessible_items(self):
        from woost.models import Publishable, IsAccessibleExpression
        return set(Publishable.select(IsAccessibleExpression(self.user)))

    def test_enabled(self):

        from woost.models import Publishable, ReadPermission

        self.everybody_role.permissions.append(
            ReadPermission(content_type = Publishable)
        )

        a = Publishable()
        a.enabled = True
        a.insert()

        b = Publishable()
        b.enabled = False
        b.insert()

        c = Publishable()
        c.enabled = True
        c.insert()

        d = Publishable()
        d.enabled = False
        d.insert()

        assert self.accessible_items() == set([a, c])

    def test_enabled_translations(self):

        from cocktail.translations import language_context
        from woost.models import (
            Publishable,
            ReadPermission,
            ReadTranslationPermission
        )

        self.everybody_role.permissions.append(
            ReadPermission(content_type = Publishable)
        )

        self.everybody_role.permissions.append(ReadTranslationPermission())

        self.config.languages = ["en"]
        self.config.published_languages = []

        with language_context("en"):
            a = Publishable()
            a.per_language_publication = True
            a.new_translation("en")
            a.insert()

            b = Publishable()
            b.per_language_publication = True
            b.new_translation("en")
            b.enabled_translations.remove("en")
            b.insert()

            c = Publishable()
            c.per_language_publication = True
            c.new_translation("en")
            c.enabled_translations.add("en")
            c.insert()

            d = Publishable()
            d.per_language_publication = True
            d.new_translation("de")
            d.insert()

            e = Publishable()
            e.per_language_publication = False
            e.enabled = True
            e.insert()

            accessible = self.accessible_items()
            print a in accessible
            print b in accessible
            print c in accessible
            print d in accessible
            print e in accessible

            assert self.accessible_items() == set([a, c, e])

            self.config.published_languages = ["en"]
            assert self.accessible_items() == set([a, c, e])

        with language_context("de"):
            self.config.published_languages = ["de"]
            assert self.accessible_items() == set([d, e])

    def test_translation_permitted(self):

        from cocktail.translations import language_context
        from woost import app
        from woost.models import (
            Publishable,
            ReadPermission,
            ReadTranslationPermission
        )

        app.user = self.user

        self.everybody_role.permissions.append(
            ReadPermission(content_type = Publishable)
        )

        self.everybody_role.permissions.append(
            ReadTranslationPermission(
                matching_languages = ["ca", "es"]
            )
        )

        self.everybody_role.permissions.append(
            ReadTranslationPermission(
                matching_languages = ["en"],
                authorized = False
            )
        )

        self.config.languages = ["ca", "es", "en"]
        self.config.published_languages = []

        a = Publishable()
        a.per_language_publication = True
        a.new_translation("ca")
        a.insert()

        b = Publishable()
        b.per_language_publication = False
        b.insert()

        with language_context("ca"):
            assert a.is_accessible()
            assert b.is_accessible()
            assert self.accessible_items() == set([a, b])

        for language in "es", "en", "de":
            with language_context(language):
                assert not a.is_accessible()
                assert b.is_accessible()
                assert self.accessible_items() == set([b])

    def test_current(self):

        from woost.models import Publishable, ReadPermission
        from datetime import datetime, timedelta

        self.everybody_role.permissions.append(
            ReadPermission(content_type = Publishable)
        )

        now = datetime.now()

        a = Publishable()
        a.enabled = True
        a.insert()

        b = Publishable()
        b.enabled = True
        b.start_date = now
        b.end_date = now + timedelta(days = 1)
        b.insert()

        c = Publishable()
        c.enabled = True
        c.start_date = now + timedelta(days = 1)
        c.insert()

        d = Publishable()
        d.enabled = True
        d.end_date = now - timedelta(days = 1)
        d.insert()

        assert self.accessible_items() == set([a, b])

    def test_allowed(self):

        from woost.models import Publishable, ReadPermission

        a = Publishable()
        a.enabled = True
        a.insert()

        b = Publishable()
        b.enabled = True
        b.insert()

        self.everybody_role.permissions.append(
            ReadPermission(
                content_type = Publishable,
                content_expression =
                    "items.add_filter(cls.id.not_equal(%d))" % b.id
            )
        )

        assert self.accessible_items() == set([a])




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.pkgutils import resolve
from cocktail.events import event_handler
from cocktail import schema
from woost import app
from woost.models.rendering.renderer import Renderer


class IconRenderer(Renderer):

    instantiable = True

    members_order = [
        "icon_size",
        "icon_resolver_expression"
    ]

    icon_size = schema.Integer(
        required = True,
        enumeration = [16, 32],
        edit_control = "cocktail.html.RadioSelector"
    )

    icon_resolver_expression = schema.String()

    def can_render(self, item, **parameters):
        return True

    def render(self, item, **parameters):
        return self.icon_resolver.find_icon_path(item, self.icon_size)

    @property
    def icon_resolver(self):
        try:
            return self._v_icon_resolver
        except AttributeError:
            expr = self.icon_resolver_expression
            icon_resolver = resolve(expr) if expr else app.icon_resolver
            self._v_icon_resolver = icon_resolver
            return icon_resolver

    @event_handler
    def handle_changed(cls, e):
        if e.member is cls.icon_resolver_expression:
            e.source._v_icon_resolver = None




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import cherrypy
from cocktail.translations import (
    get_language,
    set_language,
    clear_fallback_languages,
    set_fallback_languages
)
from cocktail.controllers import try_decode, context
from cocktail.controllers.parameters import set_cookie_expiration
from woost import app
from woost.models import Configuration


class LanguageScheme(object):

    cookie_duration = 60 * 60 * 24 * 15 # 15 days

    def process_request(self, path):

        self.setup_language_fallback_policy()

        if path:
            language = self.uri_component_to_language(path[0])
            if language in Configuration.instance.languages:
                path.pop(0)
            else:
                language = None
        else:
            language = None

        cherrypy.request.language_specified = (language is not None)

        if language is None:
            language = get_language() or self.infer_language()

        cherrypy.response.cookie["language"] = language
        cookie = cherrypy.response.cookie["language"]
        cookie["path"] = "/"
        set_cookie_expiration(cookie, seconds = self.cookie_duration)

        set_language(language)

    def infer_language(self):

        # Check for a language preference in a cookie
        cookie = cherrypy.request.cookie.get("language")

        if cookie:
            return cookie.value

        config = Configuration.instance

        # Check the 'Accept-Language' header sent by the client
        if config.get_setting("heed_client_language"):
            accept_language = cherrypy.request.headers.get("Accept-Language", None)

            if accept_language:
                available_languages = \
                    Configuration.instance.get_enabled_languages()
                best_language = None
                best_score = None

                for chunk in accept_language.split(","):
                    chunk = chunk.strip()
                    score = 1.0
                    chunk_parts = chunk.split(";")

                    if len(chunk_parts) > 1:
                        language = chunk_parts[0]
                        for part in chunk_parts[1:]:
                            if part.startswith("q="):
                                try:
                                    score = float(part[2:])
                                except TypeError:
                                    pass
                                else:
                                    break
                    else:
                        language = chunk

                    language = language.split('-', 1)[0]

                    if (
                        score
                        and language in available_languages
                        and (best_language is None or score > best_score)
                    ):
                        best_language = language
                        best_score = score

                if best_language:
                    return best_language

        # Fall back to the site's default language
        return config.get_setting("default_language")

    def translate_uri(self, path = None, language = None):

        current_language = get_language()

        if language is None:
            language = current_language

        if path is None:
            is_current_publishable = True
            path = cherrypy.request.path_info
            qs = cherrypy.request.query_string
        else:
            is_current_publishable = False
            qs = u""

        if isinstance(path, str):
            path = try_decode(path)

        if isinstance(qs, str):
            qs = try_decode(qs)

        if is_current_publishable and language != current_language:
            publishable = app.original_publishable or app.publishable
            current_uri = publishable.get_uri(
                language = current_language,
                encode = False
            )
            translation_uri = publishable.get_uri(language = language)
            path = path.replace(current_uri, translation_uri)
        else:
            path_components = path.strip("/").split("/")
            if (
                path_components
                and path_components[0] in Configuration.instance.languages
            ):
                path_components.pop(0)

            path_components.insert(0, self.language_to_uri_component(language))
            path = u"/" + u"/".join(path_components)

        return path + (u"?" + qs if qs else u"")

    def language_to_uri_component(self, language):
        return language

    def uri_component_to_language(self, uri_component):
        return uri_component

    def setup_language_fallback_policy(self):
        clear_fallback_languages()
        for language, fallback_languages \
        in Configuration.instance.fallback_languages:
            set_fallback_languages(language, fallback_languages)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import event_handler
from cocktail.pkgutils import resolve
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers import (
    FormProcessor,
    Form,
    request_property
)
from woost import app
from woost.models import changeset_context
from woost.controllers.notifications import Notification
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from .transitionpermission import TranslationWorkflowTransitionPermission


class TranslationWorkflowTransitionController(
    FormProcessor,
    BaseBackOfficeController
):
    is_transactional = True
    view_class = (
        "woost.extensions.translationworkflow."
        "TranslationWorkflowTransitionView"
    )

    @event_handler
    def handle_before_request(cls, e):
        controller = e.source
        transition = controller.transition
        user = app.user
        for request in controller.requests:
            user.require_permission(
                TranslationWorkflowTransitionPermission,
                translation_request = request,
                transition = transition
            )

    @request_property
    def requests(self):
        return self.stack_node.requests

    @request_property
    def transition(self):
        return self.stack_node.transition

    class TransitionForm(Form):

        @request_property
        def transition_setup(self):
            return (
                resolve(self.controller.transition.transition_setup_class)
                (self.controller.transition, self.controller.requests)
            )

        @request_property
        def schema(self):
            return self.transition_setup.transition_schema

        def submit(self):
            Form.submit(self)

            transition = self.controller.transition
            transition_data = self.data

            with changeset_context(author = app.user) as changeset:
                for request in self.controller.requests:
                    transition.execute(request, transition_data)
                    changeset.changes[request.id].is_explicit_change = True

        def after_submit(self):
            Notification(
                translations(
                    "woost.extensions.translationworkflow."
                    "transition_executed_notice",
                    transition = self.controller.transition,
                    requests = self.controller.requests
                ),
                "success"
            ).emit()
            self.controller.go_back()

    @request_property
    def output(self):
        output = BaseBackOfficeController.output(self)
        output["requests"] = self.requests
        output["transition"] = self.transition
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.iteration import first
from cocktail import schema
from woost.models import Block, Publishable, File
from woost.models.rendering import ImageFactory


class NewsletterContent(Block):

    type_group = "blocks.newsletter"

    members_order = [
        "view_class",
        "text",
        "image",
        "image_alignment",
        "image_factory",
        "link"
    ]

    view_class = schema.String(
        shadows_attribute = True,
        required = True,
        default = "woost.extensions.newsletters.NewsletterContentView",
        enumeration = [
            "woost.extensions.newsletters.NewsletterContentView"
        ],
        text_search = False,
        member_group = "content"
    )

    text = schema.HTML(
        edit_control = "woost.extensions.newsletters.NewsletterRichTextEditor",
        translated = True,
        spellcheck = True,
        member_group = "content"
    )

    image = schema.Reference(
        type = File,
        related_end = schema.Collection(),
        member_group = "content"
    )

    image_alignment = schema.String(
        required = True,
        enumeration = [
            "image_left",
            "image_right",
            "image_top"
        ],
        text_search = False,
        member_group = "content"
    )

    image_factory = schema.Reference(
        type = ImageFactory,
        related_end = schema.Collection(),
        relation_constraints = {
            "applicable_to_newsletters": True
        },
        edit_control = "cocktail.html.DropdownSelector",
        member_group = "content"
    )

    link = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.text = self.text
        view.image = self.image
        view.image_alignment = self.image_alignment
        view.image_factory = self.image_factory
        view.link = self.link




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import OrderedSet
from cocktail.translations import translations
from cocktail import schema
from cocktail.schema import expressions as expr
from cocktail.schema.io import MSExcelExporter, MSExcelColumn
from cocktail.html import templates
from woost import app
from woost.models import (
    Item,
    Role,
    User,
    LocaleMember
)
from woost.views.htmldiff import iter_diff_rows
from .state import TranslationWorkflowState
from .translationagency import TranslationAgency
from .utils import (
    get_models_included_in_translation_workflow,
    member_is_included_in_translation_workflow
)


class TranslationWorkflowRequestValues(schema.Mapping):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("searchable", False)
        kwargs.setdefault(
            "display",
            "woost.extensions.translationworkflow.TranslationWorkflowTable"
        )
        schema.Mapping.__init__(self, *args, **kwargs)


class TranslationWorkflowRequest(Item):

    type_group = "translation_workflow"
    instantiable = False
    backoffice_card_view = \
        "woost.extensions.translationworkflow.TranslationWorkflowRequestCard"
    edit_node_class = (
        "woost.extensions.translationworkflow.requesteditnode."
        "TranslationWorkflowRequestEditNode"
    )
    edit_form = (
        "woost.extensions.translationworkflow."
        "TranslationWorkflowRequestForm"
    )

    groups_order = [
        "translation_request.info",
        "translation_request.translated_values",
        "translation_request.changelog",
        "administration"
    ]

    members_order = [
        "translated_item",
        "source_language",
        "target_language",
        "state",
        "assigned_agency",
        "assigned_translator",
        "comments",
        "translated_values"
    ]

    translated_item = schema.Reference(
        type = Item,
        indexed = True,
        bidirectional = True,
        text_search = True,
        editable = schema.NOT_EDITABLE,
        relation_constraints = lambda ctx: [
            expr.IsInstanceExpression(
                expr.Self,
                get_models_included_in_translation_workflow()
            )
        ],
        member_group = "translation_request.info"
    )

    source_language = LocaleMember(
        required = True,
        indexed = True,
        editable = schema.NOT_EDITABLE,
        member_group = "translation_request.info"
    )

    target_language = LocaleMember(
        required = True,
        indexed = True,
        editable = schema.NOT_EDITABLE,
        member_group = "translation_request.info"
    )

    state = schema.Reference(
        type = TranslationWorkflowState,
        required = True,
        related_end = schema.Collection(),
        indexed = True,
        default = schema.DynamicDefault(
            lambda: TranslationWorkflowState.get_instance(
                qname = "woost.extensions.translationworkflow.states.pending"
            )
        ),
        search_control = "cocktail.html.DropdownSelector",
        editable = schema.NOT_EDITABLE,
        searchable = False,
        listed_by_default = False,
        member_group = "translation_request.info"
    )

    assigned_agency = schema.Reference(
        type = TranslationAgency,
        related_end = schema.Collection(),
        indexed = True,
        editable = schema.NOT_EDITABLE,
        member_group = "translation_request.info"
    )

    assigned_translator = schema.Reference(
        type = User,
        related_end = schema.Collection(),
        indexed = True,
        editable = schema.NOT_EDITABLE,
        edit_control = "cocktail.html.Autocomplete",
        search_control = "cocktail.html.Autocomplete",
        member_group = "translation_request.info"
    )

    comments = schema.Collection(
        items = "woost.extensions.translationworkflow.comment."
                "TranslationWorkflowComment",
        bidirectional = True,
        integral = True,
        editable = schema.NOT_EDITABLE,
        member_group = "translation_request.info"
    )

    translated_values = TranslationWorkflowRequestValues(
        member_group = "translation_request.translated_values"
    )

    def apply_translated_values(self):
        language = self.target_language
        for key, value in self.translated_values.iteritems():
            self.translated_item.set(key, value, language)

    def __translate__(self, language, **kwargs):
        if self.source_language and self.target_language:
            return translations(
                "woost.extensions.translationworkflow.request."
                "TranslationWorkflowRequest-instance",
                language,
                instance = self,
                referer = kwargs.get("referer")
            )
        return Item.__translate__(self, language, **kwargs)

    @classmethod
    def backoffice_listing_default_tab(cls):
        for role in app.user.iter_roles():
            default = role.translation_workflow_default_state
            if default is not None:
                return str(default.id)

        return "all"

    @classmethod
    def backoffice_listing_tabs(cls):

        states = OrderedSet()
        for role in app.user.iter_roles():
            states.extend(role.translation_workflow_relevant_states)

        if not states:
            yield (
                "all",
                translations("TranslationWorkflowRequest.tabs.all"),
                None,
                {"state": None}
            )
            states = TranslationWorkflowState.select()

        if states:
            for state in states:
                yield (
                    str(state.id),
                    state.plural_title,
                    cls.state.equal(state),
                    {"state": state}
                )


class MSExcelTranslationWorkflowSourceValuesColumn(MSExcelColumn):

    def get_cell_value(self, obj):
        return [
            obj.translated_item.get(member, language = obj.source_language)
            for member in obj.translated_item.__class__.ordered_members()
            if member_is_included_in_translation_workflow(member)
        ]


class MSExcelTranslationWorkflowTargetValuesColumn(MSExcelColumn):

    def get_cell_value(self, obj):
        return [
            obj.translated_values.get(member.name)
            for member in obj.translated_item.__class__.ordered_members()
            if member_is_included_in_translation_workflow(member)
        ]


class TranslationWorkflowRequestMSExcelExporter(MSExcelExporter):

    def get_member_columns(self, member, languages):
        if member.name == "translated_values":
            yield MSExcelTranslationWorkflowSourceValuesColumn(
                self,
                heading = translations("TranslationWorkflowRequest.msexcel.source")
            )
            yield MSExcelTranslationWorkflowTargetValuesColumn(
                self,
                heading = translations("TranslationWorkflowRequest.msexcel.target")
            )
        else:
            for column in MSExcelExporter.get_member_columns(
                self,
                member,
                languages
            ):
                yield column


TranslationWorkflowRequest.backoffice_msexcel_exporter = \
    TranslationWorkflowRequestMSExcelExporter()


@iter_diff_rows.implementation_for(TranslationWorkflowRequestValues)
def iter_diff_rows(self, language, source, target, source_value, target_value):

    keys = set()
    translated_item = schema.get(source, "translated_item")

    if translated_item is not None:

        translated_model = type(translated_item)

        if source_value is not None:
            keys.update(source_value)

        if target_value is not None:
            keys.update(target_value)

        for key in keys:

            if source_value is None:
                source_subvalue = None
            else:
                source_subvalue = source_value.get(key)

            if target_value is None:
                target_subvalue = None
            else:
                target_subvalue = target_value.get(key)

            if source_value != target_value:
                row = templates.new("woost.views.MemberDiffRow")
                row.diff_member = translated_model.get_member(key)
                row.diff_source = source
                row.diff_target = target
                row.diff_source_value = source_subvalue
                row.diff_target_value = target_subvalue
                yield row




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Configuration
from woost.extensions.audio.audiodecoder import AudioDecoder
from woost.extensions.audio.audioencoder import AudioEncoder

pos = Configuration.groups_order.index("media.images")
Configuration.groups_order.insert(pos + 1, "audio")
Configuration.members_order += ["audio_decoders", "audio_encoders"]

Configuration.add_member(
    schema.Collection("audio_decoders",
        items = schema.Reference(type = AudioDecoder),
        related_end = schema.Reference(),
        integral = True,
        member_group = "media.audio"
    )
)

Configuration.add_member(
    schema.Collection("audio_encoders",
        items = schema.Reference(type = AudioEncoder),
        related_end = schema.Reference(),
        integral = True,
        member_group = "media.audio"
    )
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import extend
from cocktail import schema
from cocktail.translations import translations
from cocktail.persistence import PersistentList
from cocktail.controllers import request_property
from woost.models import Block, EmailTemplate, Publishable
from .fields import FieldSet, Field, OptionsFieldOption


class FormBlock(Block):

    type_group = "blocks.forms"

    groups_order = list(Block.groups_order)
    groups_order.insert(1, "form")

    members_order = [
        "field_set",
        "agreements",
        "notification_receivers",
        "email_messages",
        "redirection",
        "submit_code",
        "after_submit_code",
        "view_class"
    ]

    default_controller = "woost.extensions.forms.formcontroller.FormController"

    field_set = schema.Reference(
        type = FieldSet,
        related_end = schema.Collection(),
        required = True,
        member_group = "form"
    )

    agreements = schema.Collection(
        items = "woost.extensions.forms.formagreement.FormAgreement",
        bidirectional = True,
        member_group = "form"
    )

    notification_receivers = schema.Collection(
        items = schema.EmailAddress(),
        member_group = "form"
    )

    email_messages = schema.Collection(
        items = schema.Reference(type = EmailTemplate),
        related_end = schema.Collection(),
        default = schema.DynamicDefault(
            lambda: filter(None, [
                EmailTemplate.get_instance(
                    qname = "woost.extensions.forms.default_email_notification"
                )
            ])
        ),
        member_group = "form"
    )

    redirection = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "form"
    )

    submit_code = schema.CodeBlock(
        language = "python",
        member_group = "administration"
    )

    after_submit_code = schema.CodeBlock(
        language = "python",
        member_group = "administration"
    )

    view_class = schema.String(
        shadows_attribute = True,
        after_member = "controller",
        listed_by_default = False,
        default = "woost.extensions.forms.FormView",
        member_group = "behavior"
    )

    def __init__(self, *args, **kwargs):
        Block.__init__(self, *args, **kwargs)
        self.submitted_data = PersistentList()

    def init_view(self, view):
        Block.init_view(self, view)
        view.depends_on(Field)
        view.depends_on(OptionsFieldOption)

    @request_property
    def form_model(self):
        return self.field_set.create_form_model()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Block, Slot


class NewsletterBox(Block):

    type_group = "blocks.newsletter"

    members_order = [
        "view_class",
        "blocks"
    ]

    view_class = schema.String(
        shadows_attribute = True,
        required = True,
        default = "woost.extensions.newsletters.NewsletterBoxView",
        enumeration = [
            "woost.extensions.newsletters.NewsletterBoxView"
        ],
        text_search = False,
        member_group = "content"
    )

    blocks = Slot()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from __future__ import with_statement
import cherrypy
from ZODB.POSException import ConflictError
from cocktail.events import event_handler, when, Event
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import request_property, get_parameter
from woost import app
from woost.models import (
    Configuration,
    changeset_context,
    ChangeSet,
    restricted_modification_context,
    delete_validating,
    ReadTranslationPermission
)
from woost.controllers.backoffice.editstack import RelationNode, EditNode
from woost.controllers.backoffice.useractions import get_user_action
from woost.controllers.backoffice.basebackofficecontroller \
        import BaseBackOfficeController


class EditController(BaseBackOfficeController):

    MAX_TRANSACTION_ATTEMPTS = 3
    section = None

    saving_item = Event()

    @event_handler
    def handle_processed(cls, event):
        controller = event.source
        controller.context["parent_handler"].section_redirection()
        controller.stack_node.section = controller.section

    @request_property
    def errors(self):
        if self.action:
            return schema.ErrorList(
                self.action.get_errors(self, self.action_selection)
            )
        else:
            return []

    @request_property
    def action(self):
        return (
            self.item_action
            or self.relation_action
            or self.arbitrary_action
        )

    @request_property
    def action_selection(self):
        if self.relation_action:
            return self.relation_selection
        elif self.arbitrary_action:
            return self.arbitrary_selection

        return [self.stack_node.item]

    @request_property
    def item_action(self):
        return self._get_user_action("item_action")[0]

    @request_property
    def arbitrary_action(self):
        return self.arbitrary_action_data[0]

    @request_property
    def arbitrary_selection(self):
        return self.arbitrary_action_data[1]

    @request_property
    def arbitrary_action_data(self):
        return self._get_user_action("action")

    @request_property
    def relation_action(self):
        return self._relation_action_data[0]

    @request_property
    def relation_member(self):
        return self._relation_action_data[1]

    @request_property
    def _relation_action_data(self):
        for key, value in cherrypy.request.params.iteritems():
            if key.startswith("relation_action-"):
                member_name = key.split("-", 1)[1]

                action = get_user_action(value)
                if action and not action.enabled:
                    action = None

                member = self.stack_node.content_type.get_member(member_name)
                if not isinstance(member, (schema.Collection, schema.Reference)):
                    member = None

                return action, member

        return None, None

    @request_property
    def relation_selection(self):
        member = self.relation_member
        value = get_parameter(
            schema.Collection(
                name = "relation_selection-" + member.name,
                items = schema.Reference(type = member.related_type)
            )
        )
        if not value:
            return []
        elif self.relation_action.id == "edit":
            return value
        else:
            enum = frozenset(self.stack_node.form_data.get(member.name))
            return [item for item in value if item in enum]

    @request_property
    def submitted(self):
        return self.action or self.relation_action

    @request_property
    def ready(self):
        return self.submitted and not self.errors

    def submit(self):
        if self.action:
            self._invoke_user_action(
                self.action,
                self.action_selection
            )
        elif self.relation_action:
            self._invoke_user_action(
                self.relation_action,
                self.relation_selection
            )

    def save_item(self, close = False):

        for i in range(self.MAX_TRANSACTION_ATTEMPTS):
            user = app.user
            stack_node = self.stack_node
            item = stack_node.item
            is_new = not item.is_inserted
            changeset = None

            if not item.id:
                item.id = stack_node.item_id

            with restricted_modification_context(
                item,
                user,
                member_subset = set(
                    member
                    for member in stack_node.form_schema.iter_members()
                    if member.editable == schema.EDITABLE
                )
            ):
                with changeset_context(author = user) as changeset:
                    self._apply_changes(item)
                    change = changeset.changes.get(item.id)
                    if change is not None:
                        change.is_explicit_change = True

                    self.saving_item(change = change)
            try:
                datastore.commit()
            except ConflictError:
                datastore.abort()
                datastore.sync()
            else:
                break

        change = changeset.changes.get(item.id) if changeset else None

        # Creating a new nested object: relate the created item to its owner
        if is_new and isinstance(stack_node.parent_node, RelationNode):
            member = stack_node.parent_node.member
            parent_edit_node = stack_node.get_ancestor_node(EditNode)
            parent_edit_node.relate(member, item)

        # User notification
        stack_node.item_saved_notification(is_new, change)

        # Edit stack event
        stack_node.committed(
            user = user,
            changeset = changeset
        )

        # Application-wide event
        if change is not None:
            self.context["cms"].item_saved(
                item = item,
                user = user,
                is_new = is_new,
                change = change
            )

        # Close the current node, or redirect to it using a GET request (to
        # prevent double posts)
        if close:
            self.go_back()
        else:
            raise cherrypy.HTTPRedirect(self.edit_uri(item))

    def _apply_changes(self, item):

        # Remove those instances that have been dettached from an integral
        # reference
        @when(item.changed)
        def delete_replaced_integral_children(event):
            if (
                isinstance(event.member, schema.Reference)
                and event.member.integral
                and event.previous_value is not None
            ):
                delete_validating(event.previous_value)

        try:
            stack_node = self.stack_node
            stack_node.import_form_data(stack_node.form_data, item)
            is_new = not item.is_inserted
            item.insert()
            stack_node.saving(
                user = app.user,
                item = item,
                is_new = is_new,
                changeset = ChangeSet.current
            )
        finally:
            item.changed.remove(delete_replaced_integral_children)

    @request_property
    def tab(self):
        return cherrypy.request.params.get("tab")

    @request_property
    def output(self):
        output = BaseBackOfficeController.output(self)
        stack_node = self.stack_node
        output.update(
            edited_item = stack_node.item,
            edited_content_type = stack_node.content_type,
            errors = self.errors,
            form_schema = stack_node.form_schema,
            form_data = stack_node.form_data,
            changes = set(stack_node.iter_changes()),
            item_translations = stack_node.item_translations,
            visible_translations = stack_node.visible_translations,
            section = self.section,
            tab = self.tab
        )
        return output




#-*- coding: utf-8 -*-
u"""Declares a user action for granting access to a Twitter account.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from oauth2 import Client, Consumer, Token
import cherrypy
from urllib import urlopen, quote_plus
from urlparse import parse_qsl
from simplejson import loads
from cocktail.persistence import datastore
from cocktail.translations import translations
from cocktail.controllers import Location, session
from woost.controllers.notifications import Notification
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.twitterpublication.exceptions import TwitterAPIError
from woost.extensions.twitterpublication.twitterpublicationtarget \
    import TwitterPublicationTarget


class TwitterAuthUserAction(UserAction):

    content_type = TwitterPublicationTarget

    included = frozenset(["item_buttons"])
    excluded = frozenset(["new_item"])
    min = 1
    max = 1

    def invoke(self, controller, selection):

        target = selection[0]
        consumer = Consumer(target.app_id, target.app_secret)

        def request(client, *args, **kwargs):
            response, body = client.request(*args, **kwargs)
            if response["status"] != "200":
                raise TwitterAPIError(body)
            return body

        try:
            oauth_token = cherrypy.request.params.get("oauth_token")
            oauth_verifier = cherrypy.request.params.get("oauth_verifier")
            oauth_secret_session_key = "twitter_auth.%s.oauth_secret" % self.id

            if not oauth_token or not oauth_verifier:
                # Obtain a request token
                client = Client(consumer)

                location = Location.get_current(relative = False)
                location.query_string["item_action"] = self.id
                callback = quote_plus(str(location))

                body = request(
                    client,
                    "https://api.twitter.com/oauth/request_token"
                    "?oauth_callback=" + callback,
                    "GET"
                )
                data = dict(parse_qsl(body))
                session[oauth_secret_session_key] = data["oauth_token_secret"]

                # Redirect the user to the login form
                auth_url = (
                    "https://api.twitter.com/oauth/authorize?"
                    "oauth_token=%s"
                    "&oauth_callback=%s"
                    % (data["oauth_token"], callback)
                )
                raise cherrypy.HTTPRedirect(auth_url)
            else:
                token = Token(oauth_token, session[oauth_secret_session_key])
                token.set_verifier(oauth_verifier)
                client = Client(consumer, token)
                body = request(
                    client,
                    'https://api.twitter.com/oauth/access_token',
                    "POST"
                )
                data = dict(parse_qsl(body))
                target.auth_token = data["oauth_token"]
                target.auth_secret = data["oauth_token_secret"]
                datastore.commit()

        except TwitterAPIError, ex:
            Notification(
                translations(ex),
                category = "error",
                transient = False
            ).emit()
        else:
            Notification(
                translations(
                    "woost.extensions.twitterpublication."
                    "successful_authorization_notice"
                ),
                category = "success"
            ).emit()

TwitterAuthUserAction("twitter_auth").register()




#-*- coding: utf-8 -*-
u"""Defines the `ProductController` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.translations import translations
from cocktail.controllers import request_property, Location
from cocktail.controllers.formprocessor import FormProcessor, Form
from woost import app
from woost.models import Publishable
from woost.controllers.notifications import Notification
from woost.controllers.publishablecontroller import PublishableController
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct
from woost.extensions.ecommerce.ecommercepurchase import ECommercePurchase
from woost.extensions.ecommerce.basket import Basket


class ProductController(FormProcessor, PublishableController):

    class AddProductForm(Form):

        actions = ("add_product",)
        redirect_to_basket = False

        @request_property
        def product(self):
            return app.publishable

        @request_property
        def model(self):
            return self.product.purchase_model

        @request_property
        def adapter(self):

            adapter = Form.adapter(self)
            adapter.implicit_copy = False

            for member in self.model.get_options():
                adapter.copy(member.name)

            return adapter

        def create_instance(self):
            instance = Form.create_instance(self)
            instance.product = self.product
            return instance

        def submit(self):
            Form.submit(self)
            Basket.get().add_purchase(self.instance)
            Basket.store()

            Notification(
                translations(
                    "woost.extensions.ecommerce.product_added_notice",
                    product = self.product
                ),
                "product_added",
                transient = False
            ).emit()

            if self.redirect_to_basket:
                raise cherrypy.HTTPRedirect(
                    Publishable.require_instance(
                        qname = "woost.extensions.ecommerce.basket_page"
                    ).get_uri()
                )
            else:
                Location.get_current().go("GET")




#-*- coding: utf-8 -*-
import cherrypy
from woost.models.extension import load_extensions
import --SETUP-PACKAGE--.models
import --SETUP-PACKAGE--.views
from --SETUP-PACKAGE--.controllers import --SETUP-WEBSITE--CMSController

load_extensions()
cms = --SETUP-WEBSITE--CMSController()
application = cms.mount()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import Event, event_handler
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import request_property
from woost.models import Item, File


class TranslationWorkflowTransition(Item):

    type_group = "translation_workflow"
    visible_from_root = False

    executed = Event(
        """An event triggered when the transaction is executed."""
    )

    members_order = [
        "title",
        "source_states",
        "target_state",
        "icon",
        "relative_order",
        "requires_valid_text",
        "requires_different_state",
        "transition_setup_class",
        "transition_code"
    ]

    title = schema.String(
        required = True,
        translated = True,
        descriptive = True,
        spellcheck = True
    )

    source_states = schema.Collection(
        items = "woost.extensions.translationworkflow."
               "state.TranslationWorkflowState",
        bidirectional = True,
        related_key = "outgoing_transitions"
    )

    target_state = schema.Reference(
        type = "woost.extensions.translationworkflow."
               "state.TranslationWorkflowState",
        bidirectional = True,
        related_key = "incomming_transitions"
    )

    icon = schema.Reference(
        type = File,
        relation_constraints = {"resource_type": "image"},
        related_end = schema.Collection(),
        listed_by_default = False
    )

    relative_order = schema.Integer(
        default = 0,
        indexed = True
    )

    requires_valid_text = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False
    )

    requires_different_state = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False
    )

    transition_setup_class = schema.String(
        listed_by_default = False
    )

    transition_code = schema.CodeBlock(
        language = "python",
        listed_by_default = False
    )

    def get_representative_image(self, icon_factory = None):
        return self.icon

    def execute(self, request, data = None):

        previous_state = request.state
        request.state = self.target_state

        if self.transition_code:
            label = "%r transition_code" % self
            code = compile(self.transition_code, label, "exec")
            exec code in {
                "transition": self,
                "request": request,
                "previous_state": previous_state,
                "data": data
            }

        if request.state is None:
            raise TransitionWithNoTargetStateError(self, request)

        self.executed(request = request, data = data)


class TransitionWithNoTargetStateError(Exception):

    def __init__(self, transition, request):
        self.transition = transition
        self.request = request
        Exception.__init__(
            self,
            u"%r failed to select a target state for %r"
            % (transition, request)
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from cocktail import schema
from cocktail.html import Element
from cocktail.translations import translations
from cocktail.controllers import context
from woost import app
from woost.models.rendering import Renderer
from woost.models.utils import any_translation


class ItemLabel(Element):

    item = None
    image_factory = "backoffice_small_thumbnail.png"
    icon_visible = "auto"
    label_visible = True
    referer = None
    language_chain = None

    # Make the view usable as a member display
    def _get_value(self):
        return self.item

    def _set_value(self, value):
        self.item = value

    value = property(_get_value, _set_value)

    def should_display_icon(self):

        if self.icon_visible == "auto":

            # Models can request that their icons be always visible
            if self.item.icon_conveys_information:
                return True

            # Distinguish types
            if self.member:

                if isinstance(self.member, schema.Reference):
                    related_type = self.member.related_type
                elif isinstance(self.member, schema.Schema):
                    related_type = self.member
                else:
                    related_type = None

                if related_type is not None:
                    icons = set()

                    for cls in related_type.schema_tree():
                        if cls.icon_conveys_information:
                            return True
                        if cls.visible and cls.instantiable:
                            icons.add(app.icon_resolver.find_icon(cls, 16))
                            if len(icons) > 1:
                                return True

            # Images
            renderer = Renderer.get_instance(qname = "woost.content_renderer")
            if renderer is not None:
                if renderer.can_render(self.item):
                    return True

                image = self.item.get_representative_image()
                if image is not None and renderer.can_render(image):
                    return True

            return False

        return self.icon_visible

    def _ready(self):
        Element._ready(self)

        if self.item:
            self["draggable"] = "true"
            self["data-woost-item"] = self.item.id

            for schema in self.item.__class__.descend_inheritance(True):
                self.add_class(schema.name)

            if self.should_display_icon():
                self.add_class("with_icon")
                self.append(self.create_icon())

            if self.label_visible:
                self.text_wrapper = self.create_text_wrapper()
                self.append(self.text_wrapper)

    def create_icon(self):
        img = Element("img")
        img.add_class("icon")
        img["title"] = translations(self.item.__class__.__name__)
        get_image_uri = getattr(self.item, "get_image_uri", None)

        if get_image_uri:
            img["src"] = get_image_uri(self.image_factory)
        else:
            image_factory = self.image_factory or "default"

            if "." not in image_factory:
                from woost.models.rendering.formats import (
                    extensions_by_format,
                    default_format
                )
                extension = extensions_by_format[default_format]
                image_factory += "." + extension

            img["src"] = context["cms"].image_uri(self.item, image_factory)

        return img

    def create_text_wrapper(self):
        text_wrapper = Element("span")
        text_wrapper.add_class("text_wrapper")
        text_wrapper.append(self.get_label())
        return text_wrapper

    def get_label(self):
        return any_translation(
            self.item,
            language_chain = self.language_chain,
            referer = self.referer
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from woost.models.rendering import ImageURIRenderer
from woost.extensions.issuu.issuudocument import IssuuDocument

THUMBNAIL_URL_PATTERN = "http://image.issuu.com/%s/jpg/page_%s.jpg"


class IssuuDocumentRenderer(ImageURIRenderer):

    def can_render(self, item, **parameters):
        return isinstance(item, IssuuDocument) and item.issuu_document_id and item.thumbnail_page

    def get_item_uri(self, item):
        return THUMBNAIL_URL_PATTERN % (
            item.issuu_document_id,
            item.thumbnail_page
        )





#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models import type_groups, TypeGroup

type_groups["setup"].append(
    TypeGroup("custom_forms")
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema


class MetaTags(schema.Collection):

    def __init__(self, *args, **kwargs):
        kwargs["items"] = schema.Tuple(
            items = (
                schema.String("key", required = True),
                schema.String("value")
            ),
            request_value_separator = "="
        )
        kwargs.setdefault("searchable", False)
        kwargs.setdefault("edit_control", "cocktail.html.TextArea")
        kwargs.setdefault("request_value_separator", "\n")
        schema.Collection.__init__(self, *args, **kwargs)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.html import templates
from woost.models import Extension, extension_translations

translations.define("FormsExtension",
    ca = u"Formularis personalitzats",
    es = u"Formularios personalizados",
    en = u"Custom forms"
)

translations.define("FormsExtension-plural",
    ca = u"Formularis personalitzats",
    es = u"Formularios personalizados",
    en = u"Custom forms"
)


class FormsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet dissenyar formularis des del gestor.""",
            "ca"
        )
        self.set("description",
            u"""Permite diseñar formularios des del gestor.""",
            "es"
        )
        self.set("description",
            u"""Design forms directly from the backoffice.""",
            "en"
        )

    def _load(self):

        from woost.extensions.forms import (
            strings,
            formblock,
            formagreement,
            fields,
            typegroups,
            exportformdataaction
        )

        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController

        from woost.extensions.forms.exportformdatacontroller \
            import ExportFormDataController

        BackOfficeController.export_form_data = ExportFormDataController

    def _install(self):
        self.create_default_email_notification()

    def create_default_email_notification(self):

        from woost.models import EmailTemplate, Configuration
        from woost.extensions.forms import installationstrings

        try:
            default_host = Configuration.instance.websites[0].hosts[0]
        except:
            sender = None
        else:
            pos = default_host.find(":")
            if pos != -1:
                default_host = default_host[:pos]
            parts = default_host.split(".")
            if len(parts) > 2:
                default_host = ".".join(parts[-2:])
            sender = "'info@%s'" % default_host

        email = self._create_asset(
            EmailTemplate,
            "default_email_notification",
            title = extension_translations,
            sender = sender,
            receivers = "notification_receivers",
            template_engine = "mako",
            subject = extension_translations,
            body = extension_translations,
            initialization_code =
                "from cocktail.stringutils import decapitalize\n"
                "from cocktail.translations import translations\n"
                "from cocktail.html import templates\n"
                "form_description = decapitalize(translations(block))\n"
                "table = templates.new('cocktail.html.EmailPropertyTable')\n"
                "table.schema = form_schema\n"
                "table.data = form_data\n"
                "form_contents = table.render()",
            condition = "should_send = bool(notification_receivers)"
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
import cherrypy
from woost import app


def catalog_current_state_uri():
    catalog_url = cherrypy.request.params.get("catalog_url")
    if catalog_url:
        return catalog_url
    else:
        website = app.website
        if website.ecommerce_default_catalog:
            return website.ecommerce_default_catalog.get_uri()





#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from simplejson import dumps
import cherrypy
from cocktail.translations import translations
from woost import app
from woost.models import Configuration
from woost.controllers.filecontroller import FileController
from woost.controllers.uricontroller import URIController
from woost.extensions.googleanalytics import GoogleAnalyticsExtension


class GAEventRedirection(object):

    delay = 300

    overridable_category = True
    overridable_action = True
    overridable_label = True

    default_category = None
    default_action = "open"

    template = """
        <!DOCTYPE html>
        <html>
            <head>
                <meta http-equiv="Content-Type" content="text/html;charset=utf-8">
                <title>%(title)s</title>
                %(analytics_script)s
                <script type="text/javascript">
                    window.onload = function () {
                        setTimeout(function () {
                            location.replace(%(url)s);
                        }, %(delay)d);
                    }
                </script>
            </head>
            <body>
                %(body)s
            </body>
        </html>
        """

    def __init__(self, controller_class):

        base_impl = controller_class.__call__

        def __call__(controller, **kwargs):

            publishable = app.publishable

            if self.event_tracking_is_enabled(controller, publishable):

                cherrypy.response.headers["Content-Type"] = "text/html"

                url = self.get_url(controller, publishable)
                category = self.get_category(controller, publishable)
                action = self.get_action(controller, publishable)
                label = self.get_label(controller, publishable)
                commands = [('send', 'event', category, action, label)]

                return self.template % {
                    "title": self.get_title(controller, publishable),
                    "analytics_script":
                        GoogleAnalyticsExtension.instance.get_analytics_script(
                            publishable = publishable,
                            async = False,
                            commands = commands
                        ),
                    "url": dumps(url),
                    "delay": self.delay,
                    "body": self.get_body(controller, publishable, url = url)
                }

            # Google Analytics event tracking disabled
            else:
                return base_impl(controller, **kwargs)

        controller_class.__call__ = __call__
        controller_class.ga_event_redirection = self

    def event_tracking_is_enabled(self, controller, publishable):
        return cherrypy.request.params.get("ga_event") == "1"

    def get_url(self, controller, publishable):

        extra_params = dict(
            (key, value)
            for key, value in cherrypy.request.params.iteritems()
            if not key.startswith("ga_event")
        )

        return publishable.get_uri(parameters = extra_params)

    def get_title(self, controller, publishable):
        return translations("woost.extensions.googleanalytics.redirection_title")

    def get_body(self, controller, publishable, url):
        return translations(
            "woost.extensions.googleanalytics.redirection_body",
            url = url
        )

    def get_category(self, controller, publishable):
        return (
            self.overridable_category
            and cherrypy.request.params.get("ga_event_category")
            or self.default_category
            or publishable.__class__.__name__
        )

    def get_action(self, controller, publishable):
        return (
            self.overridable_action
            and cherrypy.request.params.get("ga_event_action")
            or self.default_action
        )

    def get_label(self, controller, publishable):
        return (
            self.overridable_label
            and cherrypy.request.params.get("ga_event_label")
            or str(publishable.id)
        )

GAEventRedirection(FileController)
GAEventRedirection(URIController)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re
from cocktail.translations import translations
from woost.models import Extension

translations.define("VariablesExtension",
    ca = u"Variables",
    es = u"Variables",
    en = u"Variables"
)

translations.define("StaticPubExtension-plural",
    ca = u"Variables",
    es = u"Variables",
    en = u"Variables"
)

variable_regex = re.compile("{{([a-z0-9A-Z.-_]+)}}")


class VariablesExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet aplicar patrons de substitució de text.""",
            "ca"
        )
        self.set("description",
            u"""Permite aplicar patrones de sustitución de texto.""",
            "es"
        )
        self.set("description",
            u"""Replace text patterns with centrally defined variables.""",
            "en"
        )

    def _load(self):

        from woost.extensions.variables import (
            strings,
            variable,
            migration
        )

        from woost.controllers.publishablecontroller \
            import PublishableController

        base_produce_content = PublishableController._produce_content

        def produce_content_with_variable_replacement(controller, **kwargs):
            content = base_produce_content(controller, **kwargs)
            content = self.replace_variables(content)
            return content

        PublishableController._produce_content = \
            produce_content_with_variable_replacement

    def replace_variables(self, content):
        return variable_regex.sub(self.__replacement, content)

    def __replacement(self, match):
        from woost.extensions.variables.variable import Variable
        variable = Variable.require_instance(identifier = match.group(1))
        return variable.get_value()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import Item


class MailingList(Item):

    members_order = ["title", "mailings"]

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        spellcheck = True
    )

    users = schema.Collection(
        items = "woost.models.User",
        bidirectional = True,
        listed_by_default = False
    )

    mailings = schema.Collection(
        items = "woost.extensions.mailer.mailing.Mailing",
        bidirectional = True,
        editable = schema.NOT_EDITABLE,
        listed_by_default = False
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define(
    "woost.extensions.forms.default_email_notification.title",
    ca = u"Notificació per formularis personalitzats",
    es = u"Notificación para formularios personalizados",
    en = u"Notification for custom forms",
)

translations.define(
    "woost.extensions.forms.default_email_notification.subject",
    ca = u"S'ha emplenat ${form_description}",
    es = u"Se ha rellenado ${form_description}",
    en = u"${form_description} submitted"
)

translations.define(
    "woost.extensions.forms.default_email_notification.body",
    ca = u"""${form_contents}""",
    es = u"""${form_contents}""",
    en = u"""${form_contents}"""
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import urllib2
from bs4 import BeautifulSoup
from woost.models.configuration import Configuration
from woost.models.uri import URI, File


class URLImporter(object):

    def __init__(self):
        self.__handlers = []

    @property
    def handlers(self):
        return self.__handlers

    def prepend_handler(self, handler):
        self.__handlers.insert(0, handler)

    def append_handler(self, handler):
        self.__handlers.append(handler)

    def create_publishable(self, url, languages = None, timeout = 10):

        url_import = URLImport(self, url, languages, timeout)

        for handler in self.__handlers:
            publishable = handler(url_import)
            if publishable:
                return publishable

        return None


class URLImport(object):

    title = None
    description = None
    image_url = None
    mime_type = None
    response = None
    response_content = None
    soup = None

    def __init__(self, importer, url, languages, timeout = 10):

        self.importer = importer
        self.url = url
        self.languages = languages

        try:
            self.response = urllib2.urlopen(
                url,
                timeout = timeout
            )
            self.response_content = self.response.read()

            ct = self.response.headers.get("content-type")
            self.mime_type = ct.split(";")[0]

            if self.mime_type in ("text/html", "text/xhtml"):
                self.soup = BeautifulSoup(self.response_content)
        except:
            pass
        else:
            if self.soup:

                # Title
                title_tag = self.soup.find("meta", {"property": "og:title"})
                if title_tag:
                    self.title = title_tag["content"].strip()
                else:
                    title_tag = self.soup.find("title")
                    if title_tag:
                        self.title = title_tag.text.strip()

                # Description
                desc_tag = self.soup.find("meta", {"property": "og:description"})
                if desc_tag:
                    self.description = desc_tag["content"].strip()
                else:
                    desc_tag = self.soup.find("meta", {"name": "description"})
                    if desc_tag:
                        self.description = desc_tag["content"].strip()

                # Image
                image_tag = self.soup.find("meta", {"property": "og:image"})
                if image_tag:
                    self.image_url = image_tag["content"]

    def apply_scraped_info(self, publishable, image_member = "image"):
        publishable.mime_type = self.mime_type
        self.apply_title(publishable)
        self.apply_image(publishable, image_member)

    def apply_title(self, publishable):
        languages = self.languages or Configuration.instance.languages
        for language in languages:
            publishable.set("title", self.title, language)

    def apply_image(self, publishable, image_member = "image"):
        if (
            self.image_url
            and image_member
            and hasattr(publishable, image_member)
        ):
            image = File.from_path(self.image_url)
            setattr(publishable, image_member, image)


url_importer = URLImporter()

def import_url_as_uri(url_import):
    item = URI()
    url_import.apply_scraped_info(item)
    item.uri = url_import.url
    return item

url_importer.append_handler(import_url_as_uri)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.events import event_handler
from cocktail import schema
from woost.models.item import Item
from woost.models.file import File
from woost.models.publishable import Publishable
from woost.models.metatags import MetaTags


class Website(Item):

    instantiable = True
    visible_from_root = False

    groups_order = [
        "website",
        "contact",
        "publication",
        "publication.pages",
        "publication.maintenance",
        "publication.https",
        "language",
        "services"
    ]

    members_order = [

        # website
        "site_name",
        "hosts",
        "logo",
        "icon",
        "keywords",
        "description",

        # contact
        "organization_name",
        "organization_url",
        "address",
        "town",
        "region",
        "postal_code",
        "country",
        "phone_number",
        "fax_number",
        "email",
        "technical_contact_email",

        # publication.pages
        "home",
        "login_page",
        "generic_error_page",
        "not_found_error_page",
        "forbidden_error_page",

        # publication.maintenance
        "down_for_maintenance",
        "maintenance_page",

        # publication.https
        "https_policy",
        "https_persistence",

        # meta
        "meta_tags",

        # language
        "published_languages",
        "default_language",
        "heed_client_language"
    ]

    # website
    #--------------------------------------------------------------------------
    site_name = schema.String(
        translated = True,
        descriptive = True,
        member_group = "website"
    )

    hosts = schema.Collection(
        items = schema.String(required = True),
        min = 1,
        synchronizable = False,
        member_group = "website"
    )

    icon = schema.Reference(
        type = File,
        relation_constraints = [File.resource_type.equal("image")],
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "website"
    )

    logo = schema.Reference(
        type = File,
        relation_constraints = [File.resource_type.equal("image")],
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "website"
    )

    keywords = schema.String(
        translated = True,
        spellcheck = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "website"
    )

    description = schema.String(
        translated = True,
        spellcheck = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "website"
    )

    specific_content = schema.Collection(
        items = schema.Reference(type = Publishable),
        default_type = set,
        bidirectional = True,
        editable = schema.NOT_EDITABLE,
        synchronizable = False,
        related_key = "websites"
    )

    # contact
    #--------------------------------------------------------------------------
    organization_name = schema.String(
        translated = True,
        listed_by_default = False,
        member_group = "contact"
    )

    organization_url = schema.String(
        listed_by_default = False,
        member_group = "contact"
    )

    address = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "contact",
        listed_by_default = False
    )

    town = schema.String(
        translated = True,
        member_group = "contact",
        listed_by_default = False
    )

    region = schema.String(
        translated = True,
        member_group = "contact",
        listed_by_default = False
    )

    postal_code = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    country = schema.String(
        translated = True,
        member_group = "contact",
        listed_by_default = False
    )

    phone_number = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    fax_number = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    email = schema.EmailAddress(
        member_group = "contact",
        listed_by_default = False
    )

    technical_contact_email = schema.EmailAddress(
        member_group = "contact",
        listed_by_default = False
    )

    # publication.pages
    #--------------------------------------------------------------------------
    home = schema.Reference(
        type = Publishable,
        required = True,
        related_end = schema.Reference(block_delete = True),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    login_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    generic_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    not_found_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    forbidden_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    # publication.maintenance
    #--------------------------------------------------------------------------
    down_for_maintenance = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "publication.maintenance"
    )

    maintenance_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "publication.maintenance"
    )

    # meta
    #------------------------------------------------------------------------------
    meta_tags = MetaTags(
        member_group = "meta"
    )

    # language
    #--------------------------------------------------------------------------
    published_languages = schema.Collection(
        items = schema.String(required = True),
        member_group = "language"
    )

    default_language = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "language"
    )

    heed_client_language = schema.Boolean(
        listed_by_default = False,
        member_group = "language"
    )

    # publication.https
    #--------------------------------------------------------------------------
    https_policy = schema.String(
        required = True,
        default = "never",
        enumeration = [
            "always",
            "never",
            "per_page",
        ],
        listed_by_default = False,
        member_group = "publication.https"
    )

    https_persistence = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "publication.https"
    )

    @event_handler
    def handle_inserted(cls, event):
        index = Publishable.per_website_publication_index
        website = event.source
        for publishable in website.specific_content:
            if publishable.is_inserted:
                if len(publishable.websites) == 1:
                    index.remove(None, publishable.id)
                index.add(website.id, publishable.id)

    @event_handler
    def handle_changed(cls, event):
        if event.member is cls.home:
            if event.previous_value:
                event.previous_value.websites.remove(event.source)
            if event.value:
                event.value.websites = set([event.source])




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from warnings import warn
from woost import app

def get_current_website():
    """Gets the active website for the present context.

    The concept of an 'active' website is used by the publication machinery to
    limit the availability of content based on the site that is being visited.

    @rtype: L{Website}
    """
    warn(
        "get_current_website() has been deprecated; use app.website instead",
        DeprecationWarning,
        stacklevel = 2
    )
    return app.website

def set_current_website(website):
    """Sets the active website for the present context. See
    L{get_current_website} for more information.

    @param website: The website to set as active.
    @type website: L{Website}
    """
    warn(
        "set_current_website() has been deprecated; use app.website instead",
        DeprecationWarning,
        stacklevel = 2
    )
    app.website = website




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Item, Publishable, Website, get_current_website
from .utils import get_ga_value


class GoogleAnalyticsCustomDefinition(Item):

    visible_from_root = False

    members_order = [
        "title",
        "identifier",
        "definition_type",
        "enabled",
        "content_types",
        "initialization"
    ]

    title = schema.String(
        required = True,
        indexed = True,
        unique = True,
        translated = True,
        descriptive = True
    )

    identifier = schema.String()

    definition_type = schema.String(
        required = True,
        default = "dimension",
        enumeration = ["dimension", "metric"]
    )

    enabled = schema.Boolean(
        required = True,
        default = True
    )

    content_types = schema.Collection(
        items = schema.Reference(class_family = Item),
        default = [Publishable],
        min = 1
    )

    initialization = schema.CodeBlock(
        required = True,
        language = "python"
    )

    def applies(self, publishable, website = None):
        return isinstance(publishable, tuple(self.content_types))

    def apply(self, publishable, values, index = None, env = None):

        if index is None:
            from woost.models import Configuration
            defs = Configuration.instance.google_analytics_custom_definitions
            index = defs.index(self)

        context = {
            "publishable": publishable,
            "index": index,
            "value": schema.undefined,
            "undefined": schema.undefined,
            "env": {} if env is None else env
        }

        GoogleAnalyticsCustomDefinition.initialization.execute(self, context)
        index = context["index"]
        if index is not None:
            value = context["value"]
            if value is not schema.undefined:
                key = self.definition_type + str(index)
                value = get_ga_value(value)
                values[key] = value




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import re
from os.path import splitext
from cocktail.modeling import abstractmethod, ListWrapper
from cocktail.stringutils import normalize
from cocktail.translations import translations, get_language
from cocktail.controllers import resolve_object_ref
from woost import app
from woost.models import Publishable


class URLResolver(object):

    _url_schemes = None

    def __init__(self):
        self._url_schemes = []
        self._url_schemes_wrapper = ListWrapper(self._url_schemes)

    @property
    def url_schemes(self):
        return self._url_schemes_wrapper

    def add_url_scheme(self, url_scheme):
        self._url_schemes.append(url_scheme)

    def remove_url_scheme(self, url_scheme):
        self._url_schemes.remove(url_scheme)

    def insert_url_scheme(self, index, url_scheme):
        self._url_schemes.insert(index, url_scheme)

    def resolve_path(self, path):
        """Determines the publishable item that matches the indicated path.

        This method identifies a matching publishable item by trying each
        url scheme defined by the site, in order. Once a scheme finds a
        matching item, the search concludes.

        See L{URLScheme.resolve_path} for more details on the resolution
        process.

        @param path: The path to evaluate; A list-like object describing a
            a path relative to the application's root.
        @type path: str list

        @return: A structure containing the matching item and its publication
            details. If no matching item can be found, None is returned
            instead.
        @rtype: L{PathResolution}
        """
        if not path:
            website = app.website
            if website:
                return PathResolution(None, website.home)
        else:
            for url_scheme in self._url_schemes:
                resolution = url_scheme.resolve_path(path)
                if resolution is not None:
                    return resolution

    def get_path(self, publishable, language = None):
        """Determines the canonical path of the indicated item.

        This method queries each url scheme defined by the site, in
        order. Once a scheme yields a matching path, the search concludes. That
        first match will be considered the item's canonical path.

        See L{URLScheme.get_path} for more details on how paths for
        publishable items are determined.

        @param publishable: The item to get the canonical path for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param language: The language to get the path in (some schemes produce
            different canonical paths for the same content in different
            languages).
        @type language: str

        @return: The publication path for the indicated item, relative to the
            application's root. If none of the site's url schemes can
            produce a suitable path for the item, None is returned instead.
        @rtype: unicode
        """
        # The path to the home page is always the application root
        if publishable.is_home_page():
            return ""

        for url_scheme in self._url_schemes:
            path = url_scheme.get_path(publishable, language)
            if path is not None:
                return path


class URLScheme(object):
    """Base class for all kinds of content url schemes."""

    @abstractmethod
    def resolve_path(self, path):
        """Determines the publishable item that matches the indicated path.

        @param path: The path to evaluate; A list-like object describing a
            a path relative to the application's root.
        @type path: str list

        @return: A structure containing the matching item and its publication
            details. If no matching item can be found, None is returned
            instead.
        @rtype: L{PathResolution}
        """

    @abstractmethod
    def get_path(self, publishable, language):
        """Determines the canonical path of the indicated item.

        @param publishable: The item to get the canonical path for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param language: The language to get the path in. Some schemes produce
            different canonical paths for the same content in different
            languages.
        @type language: str

        @return: The publication path for the indicated item, relative to the
            application's root. If none of the site's publishing schemes can
            produce a suitable path for the item, None is returned instead.
        @rtype: unicode
        """

    def get_publishable_file_extension(self, publishable):
        return getattr(publishable, "file_extension", None)


class PathResolution(object):
    """A structure that provides publication information for an item.

    The L{URLScheme.resolve_path} and L{Configuration.resolve_path}
    methods use this class to wrap their return values.

    @var scheme: The publishing scheme used to resolve the publication details
        for the indicated path. This will only be set when calling
        L{Configuration.resolve_path}, to enable the caller to discern which of
        all the registered schemes handled the path's resolution.
    @type scheme: L{URLScheme}

    @var item: The publishable item that matches the indicated path.
    @type item: L{Publishable<woost.models.publishable.Publishable>}

    @var matching_path: The part of the evaluated path that was used by the
        publishing scheme to determine the matching item.
    @type matching_path: list

    @var extra_path: Additional path components that were not used by the
        publishing scheme when determining the path's matching item. This
        will be relayed to the item's handler when invoking the dispatching
        machinery.
    @type extra_path: list
    """
    scheme = None
    item = None
    matching_path = []
    extra_path = []

    def __init__(self,
        scheme,
        item,
        matching_path = None,
        extra_path = None):

        self.scheme = scheme
        self.item = item
        self.matching_path = matching_path if matching_path is not None else []
        self.extra_path = extra_path if extra_path is not None else []


class HierarchicalURLScheme(URLScheme):
    """A url scheme that publishes pages following a name hierarchy.

    The name hierarchy is defined by the L{path<woost.models.Page.path>},
    and L{parent<woost.models.publishable.Publishable.parent>} members of the
    L{Publishable<woost.models.publishable.Publishable>} class.
    """

    def resolve_path(self, path):

        remaining_path = list(path)
        extra_path = []

        while remaining_path:
            page = Publishable.get_instance(
                full_path = u"/".join(remaining_path)
            )
            if page:
                return PathResolution(
                    self,
                    page,
                    remaining_path,
                    extra_path
                )
            extra_path.append(remaining_path.pop())

    def get_path(self, publishable, language):
        return publishable.full_path


class IdURLScheme(URLScheme):
    """A url scheme that publishes items based on their id."""

    include_file_extensions = True

    def resolve_path(self, path):
        if path:
            if path:
                ref, ext = splitext(path[0])
                try:
                    id = int(ref)
                except:
                    pass
                else:
                    publishable = Publishable.get_instance(id)

                    if ext and (
                        not self.include_file_extensions
                        or ext != self.get_publishable_file_extension(publishable)
                    ):
                        return None

                    if publishable is not None:
                        return PathResolution(
                            self,
                            publishable,
                            [path[0]],
                            path[1:]
                        )

    def get_path(self, publishable, language):
        path = str(publishable.id)

        if self.include_file_extensions:
            ext = self.get_publishable_file_extension(publishable)
            if ext:
                path += ext

        return path


class DescriptiveIdURLScheme(URLScheme):
    """A url scheme that combines a unique identifier and a descriptive
    text fragment.

    @ivar word_separator: The character used to separate words on an item's
        title when crafting its descriptive fragment.
    @type word_separator: str

    @ivar id_regexp: The regular expression used to extract the unique
        identifier from an URI. The expression must define an 'id' named group.
    @type id_regexp: regular expression

    @param format: A python formatting string, used for composing an item's
        URI from its unique identifier and flattened title. Receives 'title'
        and 'id' parameters.
    @type format: str
    """
    id_separator = "_"
    word_separator = "-"
    id_regexp = re.compile(r"(.+_)?(?P<id>[^.]+)(?P<ext>\.[a-zA-Z0-9]+)?$")
    title_splitter_regexp = re.compile(r"\W+", re.UNICODE)
    format = "%(title)s%(separator)s%(id)d"
    normalize = False
    include_file_extensions = True

    _uri_encodings = ["utf-8", "iso-8859-1"]

    def resolve_path(self, path):

        if path:

            ref = path[0]

            # Try to decode the supplied URI using a selection of different
            # string encodings
            if not isinstance(ref, unicode):
                for encoding in self._uri_encodings:
                    try:
                        ref = ref.decode(encoding)
                    except UnicodeDecodeError, ex:
                        pass
                    else:
                        break
                else:
                    raise ex

            # Discard descriptive text
            match = self.id_regexp.match(ref)

            if match is None:
                return None
            else:
                id = match.group("id")

            publishable = resolve_object_ref(Publishable, id)

            if publishable is not None:

                # A file extension was provided, but either the scheme doesn't
                # accept them, or the selected item doesn't match the requested
                # file type: 404
                try:
                    ext = match.group("ext")
                except:
                    ext = None

                if ext and (
                    not self.include_file_extensions
                    or ext != self.get_publishable_file_extension(publishable)
                ):
                    return None

                return PathResolution(
                    self,
                    publishable,
                    [path[0]],
                    path[1:]
                )

    def get_path(self, publishable, language):

        if not language:
            language = get_language()

        if language:
            title = translations(publishable, language)
        else:
            title = None

        if title:
            if self.normalize:
                title = normalize(title)

            title = self.title_splitter_regexp.sub(
                self.word_separator,
                title
            )
            title = title.lower()
            ref = self.format % {
                "title": title,
                "id": publishable.id,
                "separator": self.id_separator
            }
        else:
            ref = str(publishable.id)

        if self.include_file_extensions:
            ext = self.get_publishable_file_extension(publishable)
            if ext:
                ref += ext

        return ref




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models.rendering import ImageFactory

ImageFactory.add_member(
    schema.Boolean("applicable_to_newsletters",
        required = True,
        default = False,
        after_member = "applicable_to_blocks"
    )
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail import schema
from cocktail.translations import translations
from woost import app
from woost.models import (
    Publishable,
    Controller,
    File,
    Slot
)


class ECommerceProduct(Publishable):

    instantiable = False
    type_group = "ecommerce"

    members_order = [
        "title",
        "image",
        "description",
        "price",
        "weight",
        "purchase_model",
        "purchases"
    ]

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(
            qname = "woost.extensions.ecommerce.product_controller"
        )
    )

    title = schema.String(
        indexed = True,
        normalized_index = True,
        unique = True,
        descriptive = True,
        translated = True,
        required = True,
        spellcheck = True,
        member_group = "product_data"
    )

    image = schema.Reference(
        type = File,
        related_end = schema.Collection(),
        relation_constraints = {"resource_type": "image"},
        member_group = "product_data"
    )

    description = schema.String(
        translated = True,
        edit_control = "woost.views.RichTextEditor",
        member_group = "product_data",
        spellcheck = True,
        listed_by_default = False
    )

    price = schema.Money(
        required = True,
        indexed = True,
        default = Decimal("0"),
        member_group = "product_data"
    )

    weight = schema.Decimal(
        translate_value = lambda value, language = None, **kwargs:
            "" if not value else "%s Kg" % translations(value, language),
        member_group = "product_data"
    )

    purchase_model = schema.Reference(
        class_family = "woost.extensions.ecommerce.ecommercepurchase."
                       "ECommercePurchase",
        default = schema.DynamicDefault(
            lambda: ECommerceProduct.purchase_model.class_family
        ),
        required = True,
        searchable = False,
        member_group = "product_data",
        listed_by_default = False
    )

    purchases = schema.Collection(
        items = "woost.extensions.ecommerce.ecommercepurchase."
                "ECommercePurchase",
        bidirectional = True,
        visible = False,
        member_group = "product_data"
    )

    blocks = Slot()

    def offers(self):
        website = app.website
        for pricing in website.ecommerce_pricing:
            if not pricing.hidden and pricing.applies_to(self):
                yield pricing




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.html import Element, templates
from woost.models import Publishable, News
from woost.views.viewfactory import ViewFactory

# Text only
#------------------------------------------------------------------------------
text_only = ViewFactory()

@text_only.handler_for(Publishable)
def item_link(item, parameters):
    view = Element("a")
    view["href"] = item.get_uri()
    view.append(translations(item))
    return view

# Text and icons
#------------------------------------------------------------------------------
text_and_icon = ViewFactory()

@text_and_icon.handler_for(Publishable)
def item_link_with_icon(item, parameters):

    view = templates.new(
        "woost.extensions.newsletters.NewsletterTextAndIconLink"
    )
    view.item = item

    image_factory = parameters.get("image_factory")
    if image_factory is None:
        view.image_factory = image_factory

    return view

# Summary
#------------------------------------------------------------------------------
summary = ViewFactory()

@summary.handler_for(News)
def news_summary(news, parameters):

    view = templates.new(
        "woost.extensions.newsletters.NewsletterNewsSummary"
    )
    view.news = news

    image_factory = parameters.get("image_factory")
    if image_factory is None:
        view.image_factory = image_factory

    return view

summary.register(Publishable, "item_link", item_link)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Javier Marrero <javier.marrero@whads.com>
"""

import cherrypy
from bs4 import BeautifulSoup, Comment
from premailer import *
from cocktail.controllers import request_property, Location
from woost import app
from woost.models import ModifyPermission
from woost.controllers.documentcontroller import DocumentController


class NewsletterController(DocumentController):

    def process_newsletter_html(self, html):
        soup = BeautifulSoup(html)

        # Remove all HTML comments
        is_comment = lambda text: isinstance(text, Comment)
        for comment in soup.findAll(text = is_comment):
            comment.extract()

        # Remove all scripts
        for element in soup.findAll("script"):
            element.extract()

        html = str(soup).decode("utf-8")

        # Inline CSS & absolute URLs
        premailer = Premailer(
            html,
            base_url = unicode(Location.get_current_host())
        )
        html = premailer.transform()
        return html

    def _produce_content(self, **kwargs):
        content = DocumentController._produce_content(self, **kwargs)

        # Disable newsletter processing if we are "editing" it
        if not app.user.has_permission(
            ModifyPermission,
            target = app.publishable
        ):
            content = self.process_newsletter_html(content)

        return content




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re
from decimal import Decimal
from collections import Iterable, Mapping
from cocktail.modeling import SetWrapper
from cocktail.stringutils import normalize, html_to_plain_text
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import PersistentObject
from woost.models import Configuration

def get_ga_custom_values(source, overrides = None, env = None):

    values = {}
    config = Configuration.instance
    custom_defs = config.get_setting("google_analytics_custom_definitions")

    if isinstance(source, Mapping):
        overrides = source
        source = None

    for i, custom_def in enumerate(custom_defs):

        if not custom_def.enabled:
            continue

        if overrides is not None and custom_def.identifier:
            value = overrides.get(custom_def.identifier, schema.undefined)
            if value is not schema.undefined:
                key = custom_def.definition_type + str(i + 1)
                value = get_ga_value(value)
                values[key] = value
                continue

        if source is not None and custom_def.applies(source):
            custom_def.apply(source, values, i + 1, env = env)

    return values

_escape_ga_string_nonword_expr = re.compile("\W", re.UNICODE)
_escape_ga_string_repeat_expr = re.compile("--+")

def escape_ga_string(id):
    if id:
        id = html_to_plain_text(id)
        id = normalize(id)
        id = _escape_ga_string_nonword_expr.sub("-", id).strip("-")
        id = _escape_ga_string_repeat_expr.sub("-", id)
    return id

def get_ga_value(value, language = None):

    if language is None:
        language = Configuration.instance.google_analytics_language

    if value is None:
        return ""
    elif isinstance(value, basestring):
        return escape_ga_string(value)
    elif isinstance(value, bool):
        return translations(value, language = language)
    elif isinstance(value, (int, float, Decimal)):
        return str(value)
    elif isinstance(value, type):
        return escape_ga_string(translations(value.__name__, language = language))
    elif isinstance(value, (set, frozenset, SetWrapper)):
        value = map(get_ga_value, value)
        value.sort()
        return " ".join(value)
    elif isinstance(value, Iterable):
        return " ".join(get_ga_value(item) for item in value)
    elif isinstance(value, PersistentObject):
        return (
            u"%s --%d--" % (
                escape_ga_string(
                    translations(
                        value,
                        language = language,
                        discard_generic_translation = True
                    )
                ),
                value.id
            )
        ).strip()
    else:
        raise ValueError(
            "%r is not a valid value for get_ga_value()"
            % value
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from xml.sax import parseString
from xml.sax.handler import ContentHandler
from urllib import urlopen, quote_plus
from cocktail.modeling import ListWrapper
from cocktail.translations import get_language

URL_TEMPLATE = (
    "http://www.google.com/cse"
    "?cx=%(search_engine_id)s"
    "&query=%(query)s"
    "&start=%(page)d"
    "&num=%(page_size)d"
    "&lr=%(language)s"
    "&filter=%(filter)d"
    "&client=google-csbe"
    "&output=xml_no_dtd"
    "&ie=utf8"
    "&oe=utf8"
)

def google_cse_search(
    search_engine_id,
    query,
    page = 0,
    page_size = 10,
    language = None,
    filter = True,
    results_per_page = None,
    parser = None):
    """Returns Google CSE results for the given query.

    @param query: The query to search.
    @type query: unicode

    @param page: The ordinal index of the results page to return, starting
        at 0.
    @type page: int

    @param page_size: The maximum number of results per page.
    @type page_size: int

    @param language: Restricts search results to matches for the given
        language. The language must be indicated using a two letter ISO
        language code.
    @type language: str

    @param filter: Indicates if Google should apply filtering over the
        search results (ie. to remove redundant matches).
    @type filter: bool

    @return: The results for the given query.
    @rtype: L{GoogleSearchResultsList}
    """
    cse_url = URL_TEMPLATE % {
        "search_engine_id": search_engine_id,
        "query": quote_plus(query.encode("utf-8")),
        "page": page * page_size,
        "page_size": page_size,
        "language": language or get_language(),
        "filter": int(bool(filter))
    }

    f = urlopen(cse_url)
    xml_response = f.read()
    f.close()

    results = []

    if parser is None:
        parser = GoogleCSEXMLParser()

    parseString(xml_response, parser)

    return GoogleSearchResultsList(
        parser.results,
        page,
        page_size,
        parser.result_count
    )


class GoogleSearchResult(object):
    title = ""
    url = ""
    context = ""
    page_map = None


class GoogleCSEXMLParser(ContentHandler):

    result_count = 0
    current_result = None
    status = None
    result_type = GoogleSearchResult

    READING_TOTAL_COUNT = 1
    READING_TITLE = 2
    READING_URL = 3
    READING_CONTEXT = 4

    def __init__(self, *args, **kwargs):
        ContentHandler.__init__(self, *args, **kwargs)
        self.results = []

    def startElement(self, name, attrs):
        if name == "M":
            self.status = self.READING_TOTAL_COUNT
        elif name == "T":
            self.status = self.READING_TITLE
        elif name == "U":
            self.status = self.READING_URL
        elif name == "S":
            self.status = self.READING_CONTEXT
        elif name == "R":
            result = self.result_type()
            self.results.append(result)
            self.current_result = result

    def endElement(self, name):
        self.status = None
        if name == "R":
            self.current_result = None

    def characters(self, content):
        if self.status == self.READING_TOTAL_COUNT:
            self.result_count = int(content)
        elif self.status == self.READING_TITLE:
            self.current_result.title += content
        elif self.status == self.READING_URL:
            self.current_result.url += content
        elif self.status == self.READING_CONTEXT:
            self.current_result.context += content
            self.current_result.context = self.current_result.context.replace('<br>','')


class PageMapGoogleCSEXMLParser(GoogleCSEXMLParser):

    current_data_object = None
    attrs_copy = None
    READING_ATTRIBUTE = "Attribute"

    def startElement(self, name, attrs):
        GoogleCSEXMLParser.startElement(self, name, attrs)
        if name == "PageMap":
            self.status = name
            self.current_result.page_map = {}
        elif name == "DataObject":
            self.status = name
            self.current_data_object = object_name = attrs["type"]
            self.current_result.page_map[object_name] = {}
        elif self.current_data_object and name == self.READING_ATTRIBUTE :
            self.status = self.READING_ATTRIBUTE
            self.attrs_copy = attrs.copy()
            self.current_result.context = ""

    def endElement(self, name):
        GoogleCSEXMLParser.endElement(self, name)
        if self.current_data_object and name == self.READING_ATTRIBUTE:
            self.current_result.page_map[self.current_data_object][self.attrs_copy.get("name")]\
            = self.current_result.context
            self.attrs_copy = None
            self.current_result.context = ""
        elif name == "DataObject":
            self.current_data_object = None

    def characters(self, content):
        GoogleCSEXMLParser.characters(self, content)
        if self.status == self.READING_ATTRIBUTE:
            self.current_result.context += content
            self.current_result.context = self.current_result.context.replace('<br>','')


class GoogleSearchResultsList(ListWrapper):

    def __init__(self, results, page, page_size, result_count):
        ListWrapper.__init__(self, results)
        self.page = page
        self.page_size = page_size
        self.result_count = result_count




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from cocktail.modeling import extend, call_base
from cocktail.translations import translations, require_language
from cocktail.html import Element, templates
from cocktail.html.hiddeninput import HiddenInput
from woost import app
from woost.models import (
    Item,
    Publishable,
    CreatePermission,
    ModifyPermission,
    DeletePermission
)
from woost.views.uigeneration import (
    backoffice_selection_control,
    backoffice_integral_selection_control
)


class ItemSelector(Element):

    existing_items_only = False
    selection_ui_generator = backoffice_selection_control
    integral_selection_ui_generator = backoffice_integral_selection_control
    descriptive_member = None
    descriptive_member_language = None

    def _build(self):

        Element._build(self)

        self.selection_wrapper = self.create_selection_wrapper()
        self.append(self.selection_wrapper)

        self.buttons = self.create_buttons()
        self.append(self.buttons)

    def _binding(self):
        if self.member is not None:
            self.selection_display = self.create_selection_display()
            self.selection_wrapper.append(self.selection_display)
            self.data_binding_delegate = self.selection_display

    def _ready(self):

        Element._ready(self)

        if (
            self.descriptive_member is None
            and self.member
            and self.member.related_type
        ):
            self.descriptive_member = \
                self.member.related_type.descriptive_member

        if self.descriptive_member:
            self.add_resource("/resources/scripts/ItemSelector.js")

            key = self.descriptive_member.name
            if self.descriptive_member.translated:
                key += "-" + require_language(self.descriptive_member_language)

            self.set_client_param("descriptiveMember", key)

        if self.member:

            user = app.user

            # New
            if (
                not self.existing_items_only
                and any(
                    user.has_permission(CreatePermission, target = cls)
                    for cls in self.member.type.schema_tree()
                )
            ):
                self.new_button = self.create_new_button()
                self.buttons.append(self.new_button)

            # Select
            if self.existing_items_only or not self.member.integral:
                self.select_button = self.create_select_button()
                self.buttons.append(self.select_button)

            if not self.existing_items_only:

                # Edit
                if any(
                    user.has_permission(ModifyPermission, target = cls)
                    for cls in self.member.type.schema_tree()
                ):
                    self.edit_button = self.create_edit_button()
                    self.buttons.append(self.edit_button)

                # Delete
                if self.member.integral:
                    if any(
                        user.has_permission(DeletePermission, target = cls)
                        for cls in self.member.type.schema_tree()
                    ):
                        self.delete_button = self.create_delete_button()
                        self.buttons.append(self.delete_button)

        if self.value is None:
            self.add_class("empty_selection")

    def create_selection_wrapper(self):
        selection_wrapper = Element()
        selection_wrapper.add_class("selection_wrapper")
        return selection_wrapper

    def create_selection_display(self):

        if self.member.integral:
            ui_generator = self.integral_selection_ui_generator
        else:
            ui_generator = self.selection_ui_generator

        selection_display = ui_generator.create_member_display(
            self.data,
            self.member,
            self.value
        )
        selection_display.add_class("selection_display")

        # Displays for integral relations will typically be read only; add a
        # hidden input to hold their value if required.
        if self.member.integral:
            element = selection_display
            while element.data_binding_delegate is not None:
                element = element.data_binding_delegate
            if not element.is_form_control:
                hidden_input = HiddenInput()
                selection_display.append(hidden_input)
                selection_display.data_binding_delegate = hidden_input

        return selection_display

    def create_buttons(self):
        buttons = Element()
        buttons.add_class("ItemSelector-buttons")
        return buttons

    def create_select_button(self):

        select_button = Element("button",
            name = "relation-select",
            type = "submit",
            class_name = "ItemSelector-button select",
            value = self.member.type.full_name + "-" + self.name
        )
        select_button.append(
            translations("woost.views.ItemSelector select")
        )
        return select_button

    def create_unlink_button(self):

        unlink_button = Element("button",
            name = "relation-unlink",
            type = "submit",
            class_name = "ItemSelector-button unlink",
            value = self.member.name
        )
        unlink_button.append(
            translations("woost.views.ItemSelector unlink")
        )
        return unlink_button

    def create_new_button(self):

        new_button = Element(class_name = "ItemSelector-button new")

        instantiable_types = set(
            content_type
            for content_type in (
                [self.member.type] + list(self.member.type.derived_schemas())
            )
            if content_type.visible
            and content_type.instantiable
            and app.user.has_permission(
                CreatePermission,
                target = content_type
            )
        )

        if len(instantiable_types) > 1:

            new_button.add_class("selector")
            label = Element("span", class_name = "label")
            new_button.append(label)

            container = Element(class_name = "selector_content")
            new_button.append(container)

            content_type_tree = templates.new("woost.views.ContentTypeTree")
            content_type_tree.root = self.member.type
            content_type_tree.filter_item = instantiable_types.__contains__

            @extend(content_type_tree)
            def create_label(tree, content_type):
                label = call_base(content_type)
                label.tag = "button"
                label["type"] = "submit"
                label["name"] = "relation-new"
                label["value"] = self.member.name + "-" + content_type.full_name
                return label

            container.append(content_type_tree)
        else:
            new_button.tag = "button"
            new_button["type"] = "submit"
            new_button["name"] = "relation-new"
            new_button["value"] = \
                self.member.name + "-" + list(instantiable_types)[0].full_name
            label = new_button

        label.append(translations("woost.views.ItemSelector new"))

        return new_button

    def create_edit_button(self):

        edit_button = Element("button",
            name = "relation-edit",
            type = "submit",
            class_name = "ItemSelector-button edit",
            value = self.member.name
        )
        edit_button.append(
            translations("woost.views.ItemSelector edit")
        )
        return edit_button

    def create_delete_button(self):

        delete_button = Element("button",
            name = "relation-unlink",
            type = "submit",
            class_name = "ItemSelector-button delete",
            value = self.member.name
        )
        delete_button.append(
            translations("woost.views.ItemSelector delete")
        )
        return delete_button




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Javier Marrero <javier.marrero@whads.com>
"""
from cocktail.translations import translations
from woost.models import (
    Configuration,
    Controller,
    Template,
    Extension,
    extension_translations,
    rendering
)

translations.define("NewslettersExtension",
    ca = u"Butlletins de correu electrònic",
    es = u"Boletines de correo electrónico",
    en = u"Newsletters"
)

translations.define("NewslettersExtension-plural",
    ca = u"Butlletins de correu electrònic",
    es = u"Boletines de correo electrónico",
    en = u"Newsletters"
)


class NewslettersExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona una eina de creació i edició de butlletins
            electrònics (newsletters).""",
            "ca"
        )
        self.set("description",
            u"""Proporciona una herramienta para la creación y edición de
            boletines electrónicos (newsletters).""",
            "es"
        )
        self.set("description",
            u"""Provides a tool to create and edit newsletters.""",
            "en"
        )

    def _load(self):

        from woost.models import TypeGroup, block_type_groups

        block_type_groups.insert(
            "before blocks.custom",
            TypeGroup("blocks.newsletter")
        )

        from woost.extensions.newsletters import (
            strings,
            imagefactory,
            newsletter,
            newslettercontent,
            newsletterbox,
            newsletterlisting
        )

        self.install()

    def _install(self):

        # Newsletter controller
        newsletter_controller = self._create_asset(
            Controller,
            "newsletter_controller",
            python_name =
                "woost.extensions.newsletters.newslettercontroller."
                "NewsletterController",
            title = extension_translations
        )

        # Newsletter template
        newsletter_view = self._create_asset(
            Template,
            "newsletter_template",
            identifier = "woost.extensions.newsletters.NewsletterLayout",
            title = extension_translations
        )

        # Image factories
        for img_factory_name, effects in (
            ("third", [rendering.Fill(width = "200", height = "100")]),
            ("half", [rendering.Fill(width = "300", height = "150")]),
            ("full", [rendering.Thumbnail(width = "600")])
        ):
            image_factory = self._create_asset(
                rendering.ImageFactory,
                "image_factories." + img_factory_name,
                identifier = "newsletter_" + img_factory_name,
                title = extension_translations,
                applicable_to_blocks = False,
                applicable_to_newsletters = True,
                effects = effects
            )
            Configuration.instance.image_factories.append(image_factory)



#-*- coding: utf-8 -*-
u"""Defines the `Basket` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.persistence import transactional
from cocktail.events import Event
from cocktail.controllers import session
from woost import app
from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder
from woost.extensions.ecommerce.ecommercepurchase import ECommercePurchase
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct


class Basket(object):

    session_key = "woost.extensions.ecommerce.basket"

    created = Event(doc = """
        An event triggered on the class when a new instance is created.

        @ivar basket: A reference to the new instance.
        @type basket: L{Basket}
        """)


    @classmethod
    def get(cls, create_new = True):
        """Obtains the shop order for the current user session.

        If the user had not started an order yet, a new one is created.

        :rtype: `~woost.extensions.ecommerce.ecommerceorder.ECommerceOrder`
        """
        order = getattr(
            cherrypy.request,
            "woost_ecommerce_order",
            None
        )

        if order is None:
            order = cls.restore()

            if order is None and create_new:
                order = ECommerceOrder()
                cls.created(basket = order)

            cherrypy.request.woost_ecommerce_order = order

        if order:
            order.customer = app.user
        return order

    @classmethod
    def pop(cls):
        """Remove the current shop order from the session."""
        order = cls.get(create_new = False)

        if order is not None:
            session.pop(cls.session_key, None)
            del cherrypy.request.woost_ecommerce_order

        return order

    @classmethod
    @transactional()
    def empty(cls):
        """Removes all products from the basket."""
        order = cls.get(create_new = False)

        if order is not None:
            for purchase in list(order.purchases):
                purchase.delete()

    @classmethod
    @transactional()
    def store(cls):

        order = cls.get()
        order.website = app.website
        order.insert()

        for purchase in order.purchases:
            purchase.insert()

        session[cls.session_key] = order.id

    @classmethod
    def restore(cls):
        session_data = session.get(cls.session_key)

        if session_data is None:
            return None
        else:
            return ECommerceOrder.get_instance(session_data)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.html import templates
from woost.views.uigeneration import backoffice_display

PropertyTable = templates.get_class("cocktail.html.PropertyTable")


class ContentPropertyTable(PropertyTable):

    base_ui_generators = [backoffice_display]




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			August 2009
"""
from cocktail.modeling import getter, cached_getter
from cocktail.schema.expressions import (
    Self,
    Constant,
    InclusionExpression,
    ExclusionExpression,
    IsInstanceExpression,
    IsNotInstanceExpression
)
from cocktail import schema
from cocktail.translations import translations
from cocktail.html import templates
from cocktail.html.datadisplay import MULTIPLE_SELECTION
from cocktail.controllers.userfilter import (
    UserFilter,
    CollectionFilter,
    user_filters_registry,
    DescendsFromFilter
)
from woost import app
from .item import Item
from .changesets import ChangeSet, ChangeSetHasChangeExpression
from .publishable import Publishable, IsPublishedExpression
from .document import Document


class IsPublishedFilter(UserFilter):

    id = "published"

    @cached_getter
    def schema(self):
        return schema.Schema()

    @cached_getter
    def expression(self):
        return IsPublishedExpression(app.user)

user_filters_registry.add(Publishable, IsPublishedFilter)


class TypeFilter(UserFilter):
    id = "type"
    operators = ["eq", "ne"]

    @cached_getter
    def schema(self):
        return schema.Schema("UserFilter", members = [
            schema.String(
                "operator",
                required = True,
                enumeration = self.operators
            ),
            schema.Collection(
                "types",
                items = schema.Reference("item_type",
                    class_family = self.content_type
                )
            ),
            schema.Boolean(
                "is_inherited"
            )
        ])

    @cached_getter
    def expression(self):

        if self.operator == "eq":
            return IsInstanceExpression(Self, tuple(self.types), self.is_inherited)
        elif self.operator == "ne":
            return IsNotInstanceExpression(Self, tuple(self.types), self.is_inherited)

user_filters_registry.add(Item, TypeFilter)

class ItemSelectorFilter(schema.Reference.user_filter):

    def search_control(self, parent, obj, member):
        control = templates.new("woost.views.ItemSelector")
        control.existing_items_only = True
        return control

schema.Reference.user_filter = ItemSelectorFilter

def _collection_search_control(self, parent, obj, member):
    control = templates.new("woost.views.ItemSelector")
    control.existing_items_only = True
    return control

CollectionFilter.search_control = _collection_search_control


class ChangeSetHasChangeFilter(UserFilter):

    id = "changeset-change"

    @cached_getter
    def schema(self):
        return schema.Schema("ChangeSetHasChangeFilter", members = [
            schema.Reference(
                "target",
                type = Item
            ),
            schema.Reference(
                "target_type",
                class_family = Item
            ),
            schema.String(
                "action",
                enumeration = ["create", "modify", "delete"],
                translate_value = lambda value, language = None, **kwargs:
                    translations(
                        "ChangeSetHasChangeFilter.action=none",
                        language,
                        **kwargs
                    )
                    if not value
                    else translations(
                        "woost %s action" % value,
                        language,
                        **kwargs
                    )
            )
        ])

    @getter
    def expression(self):
        return ChangeSetHasChangeExpression(
            self.target or self.target_type,
            self.action,
            include_implicit = False
        )

user_filters_registry.add(ChangeSet, ChangeSetHasChangeFilter)

user_filters_registry.add(Publishable, DescendsFromFilter)
user_filters_registry.set_filter_parameter(
    Publishable,
    DescendsFromFilter,
    "relation", Document.children
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from urllib import urlopen, urlencode
from simplejson import dumps, loads
from poster.encode import multipart_encode
from poster.streaminghttp import register_openers
import urllib2
register_openers()

from cocktail.translations import (
    get_language,
    translations
)
from cocktail import schema
from woost.models import Item, Configuration
from woost.extensions.opengraph import OpenGraphExtension

facebook_locales = {
    "Catalan": 1,
    "Czech": 2,
    "Welsh": 3,
    "Danish": 4,
    "German": 5,
    "English (Upside Down)": 51,
    "English (US)": 6,
    "Spanish": 23,
    "Spanish (Spain)": 7,
    "Spanish (All)": 1002,
    "Finnish": 8,
    "French (France)": 9,
    "Hungarian": 30,
    "Italian": 10,
    "Japanese": 11,
    "Korean": 12,
    "Norwegian (bokmal)": 13,
    "Dutch": 14,
    "Polish": 15,
    "Portuguese (Brazil)": 16,
    "Portuguese (Portugal)": 31,
    "Romanian": 32,
    "Russian": 17,
    "Slovak": 33,
    "Slovenian": 34,
    "Swedish": 18,
    "Thai": 35,
    "Turkish": 19,
    "Simplified Chinese (China)": 20,
    "Traditional Chinese (Hong Kong)": 21,
    "Traditional Chinese (Taiwan)": 22,
    "Afrikaans": 36,
    "Bengali": 45,
    "Bulgarian": 37,
    "Croatian": 38,
    "English (UK)": 24,
    "French (Canada)": 44,
    "Greek": 39,
    "Hindi": 46,
    "Indonesian": 25,
    "Lithuanian": 40,
    "Malay": 41,
    "Punjabi": 47,
    "Serbian": 42,
    "Filipino": 26,
    "Tamil": 48,
    "Telugu": 49,
    "Malayalam": 50,
    "Vietnamese": 27,
    "Arabic": 28,
    "Hebrew": 29
}


class FacebookPublicationTarget(Item):

    members_order = [
        "title",
        "graph_object_id",
        "administrator_id",
        "app_id",
        "app_secret",
        "auth_token",
        "languages",
        "targeting"
    ]

    visible_from_root = False

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        descriptive = True,
        spellcheck = True,
        listed_by_default = False
    )

    graph_object_id = schema.String(
        required = True
    )

    administrator_id = schema.String(
        listed_by_default = False
    )

    app_id = schema.String(
        required = True,
        listed_by_default = False
    )

    app_secret = schema.String(
        required = True,
        listed_by_default = False,
        text_search = False
    )

    auth_token = schema.String(
        editable = schema.NOT_EDITABLE,
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            translations(
                "FacebookPublicationTarget.auth_token-%s"
                % ("conceded" if value else "pending"),
                language,
                **kwargs
            )
    )

    languages = schema.Collection(
        items = schema.String(
            enumeration = lambda ctx: Configuration.instance.languages,
            translate_value = lambda value, language = None, **kwargs:
                "" if not value else translations(value)
        )
    )

    targeting = schema.CodeBlock(
        language = "python"
    )

    def publish(self, publishable):

        if self.auth_token is None:
            raise ValueError(
                "Can't publish %s to %s: missing authorization token."
                % (publishable, self)
            )

        graph_url = "https://graph.facebook.com/%s/feed" % self.graph_object_id
        post_data = self._get_publication_parameters(publishable)
        encoded_post_data = dict(
            (k, v.encode("utf-8") if isinstance(v, unicode) else v)
            for k, v in post_data.iteritems()
        )
        response = urlopen(graph_url, urlencode(encoded_post_data))
        status = response.getcode()
        if status < 200 or status > 299:
            raise FacebookPublicationError(response.read())

    def _get_publication_parameters(self, publishable):

        og = OpenGraphExtension.instance
        og_properties = og.get_properties(publishable)

        post_data = {
            "access_token": self.auth_token,
            "name": og_properties.get("og:title") or translations(publishable),
            "link": og_properties.get("og:url")
        }

        # Disregard any default image; only show an image if the published
        # content defines one
        if "og:image" not in og_properties:
            post_data["picture"] = ""

        if self.targeting:
            context = {
                "language": get_language(),
                "og_properties": og_properties,
                "publishable": publishable,
                "locales": facebook_locales,
                "include_locales": (
                    lambda included:
                        [facebook_locales[loc_name] for loc_name in included]
                ),
                "exclude_locales": (
                    lambda excluded:
                        [loc_id
                        for loc_name, loc_id in facebook_locales.iteritems()
                        if loc_name not in excluded]
                )
            }
            targeting_code = self.targeting.replace("\r\n", "\n")
            exec targeting_code in context
            targeting = context.get("targeting")
            if targeting:
                post_data["feed_targeting"] = dumps(targeting)

        return post_data

    def feed_posts(self):

        if self.auth_token is None:
            raise ValueError(
                "Can't read the posts in %s: missing authorization token."
                % self
            )

        graph_url = "https://graph.facebook.com/%s/feed/?access_token=%s" % (
            self.graph_object_id,
            self.auth_token
        )
        response = urlopen(graph_url)
        status = response.getcode()
        body = response.read()
        if status < 200 or status > 299:
            raise FacebookPublicationError(body)

        feed_data = loads(body)
        return feed_data["data"]

    def find_post(self, publishable):

        uri = self._get_publication_parameters(publishable)["link"]

        for post in self.feed_posts():
            if post.get("link") == uri:
                return post

    def publish_album(self,
        album_title,
        photos,
        album_description = None,
        photo_languages = None,
        generate_story = True):

        if self.auth_token is None:
            raise ValueError(
                "Can't publish album to %s: missing authorization token."
                % self
            )

        # Create the album
        response = urlopen(
            "https://graph.facebook.com/%s/albums" % self.graph_object_id,
            _encode_form({
                "access_token": self.auth_token,
                "name": album_title,
                "message": album_description or ""
            })
        )

        status = response.getcode()
        body = response.read()
        if status < 200 or status > 299:
            raise FacebookPublicationError(body)

        album_id = loads(body)["id"]
        photos_url = "https://graph.facebook.com/%s/photos" % album_id

        # Upload photos
        for photo in photos:

            if photo_languages is None:
                photo_languages = photo.translations.keys()
            else:
                photo_languages = list(
                    set(photo_languages) & set(photo.translations.keys())
                )

            if not photo_languages:
                photo_desc = ""
            elif len(photo_languages) == 1:
                photo_desc = translations(photo, photo_languages[0])
            else:
                photo_desc = u"\n".join(
                    u"%s: %s" % (
                        translations(lang, lang),
                        translations(photo, lang)
                    )
                    for lang in photo_languages
                )

            datagen, headers = multipart_encode({
                "access_token": self.auth_token,
                "source": open(photo.file_path),
                "message": photo_desc,
                "no_story": ("0" if generate_story else "1")
            })
            request = urllib2.Request(photos_url, datagen, headers)
            response = urllib2.urlopen(request)

            status = response.getcode()
            body = response.read()
            if status < 200 or status > 299:
                raise FacebookPublicationError(body)

        response = urlopen(
            "https://graph.facebook.com/%s/?access_token=%s" % (
                album_id,
                self.auth_token
            )
        )
        status = response.getcode()
        body = response.read()
        if status < 200 or status > 299:
            raise FacebookPublicationError(body)

        return loads(body)

def _encode_form(form):
    return urlencode(dict(
        (k, v.encode("utf-8") if isinstance(v, unicode) else v)
        for k, v in form.iteritems()
    ))


class FacebookPublicationError(Exception):

    def __init__(self, response):
        Exception.__init__(self, response)
        self.response = response




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.pkgutils import resolve
from cocktail import schema
from cocktail.persistence.query import Query
from cocktail.controllers import get_parameter, request_property
from cocktail.controllers.autocomplete import (
    MemberAutocompleteSource,
    AutocompleteController as BaseAutocompleteController,
)
from woost import app
from woost.models import (
    Item,
    Publishable,
    ReadPermission,
    PermissionExpression,
    IsAccessibleExpression
)


class CMSAutocompleteSource(MemberAutocompleteSource):

    publishable_check = False
    include_entry_type = True

    @request_property
    def items(self):
        items = MemberAutocompleteSource.items(self)
        user = app.user

        if self._listing_types:
            items = [
                item
                for item in items
                if user.has_permission(ReadPermission, target = item)
            ]
        else:
            if (
                self.publishable_check
                and issubclass(self.member.type, Publishable)
            ):
                items.add_filter(IsAccessibleExpression())
            else:
                items.add_filter(
                    PermissionExpression(
                        user,
                        ReadPermission
                    )
                )

        return items

    def get_entry(self, item):
        entry = MemberAutocompleteSource.get_entry(self, item)

        if self._listing_types:
            entry["icon"] = app.icon_resolver.find_icon_url(item, 16)
        else:
            entry["type"] = item.__class__.get_qualified_name(include_ns = True)

        return entry


class AutocompleteController(BaseAutocompleteController):

    def __init__(self, *args, **kwargs):
        BaseAutocompleteController.__init__(
            self,
            self._autocomplete_factory,
            *args,
            **kwargs
        )

    def _autocomplete_factory(self, query):
        autocomplete_class = resolve(self.member.autocomplete_class)
        return autocomplete_class(self.member, query)

    def __call__(self, member_ref, query = "", lang = None):
        self.member = get_parameter(
            schema.MemberReference(
                required = True,
                schemas = (Item,)
            ),
            source = lambda name: member_ref,
            errors = "raise"
        )
        return BaseAutocompleteController.__call__(self, query, lang)




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
import cherrypy
from cocktail.controllers import get_state
from woost import app
from woost.controllers import BaseCMSController


class URIController(BaseCMSController):

    def __call__(self, *args, **kwargs):

        if app.publishable.is_internal_content():
            parameters = get_state()
        else:
            parameters = None

        uri = app.publishable.get_uri(parameters = parameters)
        raise cherrypy.HTTPRedirect(uri)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.typegroups import type_groups, TypeGroup

type_groups["setup"].append(TypeGroup("translation_workflow"))




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import re
from warnings import warn
from decimal import Decimal
from cocktail.modeling import classgetter
from cocktail.translations import (
    translations,
    require_language,
    set_language,
    set_fallback_languages
)
from cocktail import schema
from woost import app
from .item import Item
from .slot import Slot
from .publishable import Publishable
from .website import Website
from .caching import CachingPolicy
from .rendering.renderer import Renderer
from .rendering.imagefactory import ImageFactory
from .videoplayersettings import VideoPlayerSettings
from .trigger import Trigger
from .metatags import MetaTags

try:
    from fractions import Fraction
except ImportError:
    _numerical_types = (int, float, Decimal)
else:
    _numerical_types = (int, float, Decimal, Fraction)


class Configuration(Item):

    instantiable = False
    type_group = "setup"
    skip_listing = True

    groups_order = [
        "publication",
        "publication.pages",
        "publication.maintenance",
        "meta",
        "language",
        "media.images",
        "media.video",
        "rendering",
        "services",
        "system",
        "system.smtp",
        "admin"
    ]

    members_order = [
        "websites",
        "caching_policies",
        "down_for_maintenance",
        "maintenance_page",
        "maintenance_addresses",
        "login_page",
        "generic_error_page",
        "not_found_error_page",
        "forbidden_error_page",
        "languages",
        "published_languages",
        "virtual_languages",
        "default_language",
        "fallback_languages",
        "heed_client_language",
        "backoffice_language",
        "backoffice_language_chain",
        "renderers",
        "image_factories",
        "video_player_settings",
        "timezone",
        "smtp_host",
        "smtp_user",
        "smtp_password",
        "triggers"
    ]

    @classgetter
    def instance(cls):
        return cls.get_instance(qname = "woost.configuration")

    common_blocks = Slot()

    # publication
    #--------------------------------------------------------------------------
    websites = schema.Collection(
        items = schema.Reference(type = Website),
        related_end = schema.Collection(),
        integral = True,
        member_group = "publication"
    )

    caching_policies = schema.Collection(
        items = schema.Reference(type = CachingPolicy),
        integral = True,
        related_end = schema.Reference(),
        member_group = "publication"
    )

    login_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    generic_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    not_found_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    forbidden_error_page = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "publication.pages"
    )

    down_for_maintenance = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "publication.maintenance"
    )

    maintenance_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "publication.maintenance"
    )

    maintenance_addresses = schema.Collection(
        items = schema.String(),
        listed_by_default = False,
        searchable = False,
        member_group = "publication.maintenance"
    )

    # meta
    #------------------------------------------------------------------------------
    meta_tags = MetaTags(
        member_group = "meta"
    )

    # language
    #------------------------------------------------------------------------------
    languages = schema.Collection(
        items = schema.String(
            format = "^[a-z]{2}(-[A-Z]{2})?(-[a-z]+)?$"
        ),
        min = 1,
        listed_by_default = False,
        searchable = False,
        member_group = "language"
    )

    published_languages = schema.Collection(
        items = schema.String(
            enumeration = languages,
            edit_control = "cocktail.html.TextBox"
        ),
        searchable = False,
        listed_by_default = False,
        member_group = "language"
    )

    virtual_languages = schema.Collection(
        items = schema.String(
            enumeration = languages,
            edit_control = "cocktail.html.TextBox"
        ),
        searchable = False,
        listed_by_default = False,
        member_group = "language"
    )

    default_language = schema.String(
        required = True,
        enumeration = languages,
        edit_control = "cocktail.html.TextBox",
        text_search = False,
        listed_by_default = False,
        searchable = False,
        member_group = "language"
    )

    fallback_languages = schema.Collection(
        items = (
            schema.Tuple(
                items = (
                    schema.String(enumeration = languages),
                    schema.Collection(
                        type = tuple,
                        items = schema.String(
                            enumeration = languages
                        ),
                        request_value_separator = ","
                    )
                ),
                request_value_separator = ":"
            )
        ),
        request_value_separator = "\n",
        edit_control = "cocktail.html.TextArea",
        searchable = False,
        member_group = "language"
    )

    def setup_languages(self):
        set_language(self.default_language)
        for language, fallback_languages in self.fallback_languages:
            set_fallback_languages(language, fallback_languages)

    heed_client_language = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "language"
    )

    backoffice_language = schema.String(
        required = True,
        enumeration = ["en", "es", "ca"],
        default = "en",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(value, language, **kwargs),
        listed_by_default = False,
        member_group = "language"
    )

    backoffice_language_chain = schema.Collection(
        items = schema.String(
            enumeration = languages,
            edit_control = "cocktail.html.TextBox"
        ),
        searchable = False,
        listed_by_default = False,
        member_group = "language"
    )

    # media
    #--------------------------------------------------------------------------
    renderers = schema.Collection(
        items = schema.Reference(type = Renderer),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "media.images"
    )

    image_factories = schema.Collection(
        items = schema.Reference(type = ImageFactory),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "media.images"
    )

    video_player_settings = schema.Collection(
        items = schema.Reference(type = VideoPlayerSettings),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference(),
        member_group = "media.video"
    )

    # system
    #--------------------------------------------------------------------------
    timezone = schema.String(
        required = False,
        format = re.compile(r'^["GMT"|"UTC"|"PST"|"MST"|"CST"|"EST"]{3}$|^[+-]\d{4}$'),
        text_search = False,
        member_group = "system",
        listed_by_default = False
    )

    smtp_host = schema.String(
        default = "localhost",
        listed_by_default = False,
        text_search = False,
        member_group = "system.smtp"
    )

    smtp_user = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "system.smtp"
    )

    smtp_password = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "system.smtp"
    )

    # administration
    #--------------------------------------------------------------------------
    triggers = schema.Collection(
        items = schema.Reference(type = Trigger),
        related_end = schema.Reference(),
        integral = True,
        member_group = "administration"
    )

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.__name__, language, **kwargs)

    def resolve_path(self, path):
        warn(
            "Configuration.resolve_path is deprecated, use app.url_resolver.resolve_path instead",
            DeprecationWarning,
            stacklevel = 2
        )
        return app.url_resolver.resolve_path(path)

    def get_path(self, publishable, language = None):
        warn(
            "Configuration.get_path is deprecated, use app.url_resolver.get_path instead",
            DeprecationWarning,
            stacklevel = 2
        )
        return app.url_resolver.get_path(publishable, language = language)

    def get_setting(self, key):
        """Obtains the value for the indicated configuration option.

        The method will search the active website first (see
        L{woost.Application.website}), and the site's L{Configuration}
        if there is no active website, or if the website provides no
        significant value. Significant values include numbers, booleans, and
        any object that evaluates to True on a boolean context.

        @param key: The name of the value to obtain. Must match one of the
            members of the L{Configuration} or the L{Website} models.
        @type key: str

        @return: The value for the indicated configuration option.
        """
        website = app.website

        if website is not None:
            value = getattr(website, key, None)
            if self._is_significant_setting_value(key, value):
                return value

        return getattr(self.instance, key, None)

    def _is_significant_setting_value(self, key, value):
        return value or isinstance(value, _numerical_types)

    def language_is_enabled(self, language = None):
        language_subset = self.get_setting("published_languages")
        if not language_subset:
            return True
        else:
            return require_language(language) in language_subset

    def get_enabled_languages(self):
        return self.get_setting("published_languages") or self.languages

    def connect_to_smtp(self):

        import smtplib
        smtp = smtplib.SMTP(self.get_setting("smtp_host"), smtplib.SMTP_PORT)

        user = self.get_setting("smtp_user")
        password = self.get_setting("smtp_password")

        if user and password:
            smtp.login(str(user), str(password))

        return smtp

    def get_website_by_host(self, host):

        # Specific match
        for website in self.websites:
            if host in website.hosts:
                return website

        # Partial wildcards (ie. *.foo.com)
        while True:
            pos = host.find(".")

            if pos == -1:
                break

            host = host[pos + 1:]
            wildcard = "*." + host
            for website in self.websites:
                if wildcard in website.hosts:
                    return website

        # Generic wildcard (*)
        for website in self.websites:
            if "*" in website.hosts:
                return website




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Pau Congost <pau.congost@whads.com>
"""
from cocktail.events import when
from cocktail.persistence import MigrationStep

step = MigrationStep("woost.extensions.locations: index Location.parent")

@when(step.executing)
def index_location_parent(e):
    from woost.extensions.locations.location import Location
    Location.parent.rebuild_index()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Variable
#------------------------------------------------------------------------------
translations.define("Variable",
    ca = u"Variable",
    es = u"Variable",
    en = u"Variable"
)

translations.define("Variable-plural",
    ca = u"Variables",
    es = u"Variables",
    en = u"Variables"
)

translations.define("Variable.identifier",
    ca = u"Identificador",
    es = u"Identificador",
    en = u"Identifier"
)

translations.define("Variable.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("Variable.translated_text",
    ca = u"Text traduït",
    es = u"Texto traducido",
    en = u"Translated text"
)

translations.define("Variable.dynamic_value",
    ca = u"Expressió",
    es = u"Expresión",
    en = u"Expression"
)

translations.define("Variable.expression-explanation",
    ca = u"""
        <p>
            Bloc de codi Python que permet produïr el valor de la variable
            programàticament.
        </p>
        <details>
            <summary>Detalls</summary>
            <p>
                Si es deixa en blanc, el valor obtingut a partir dels camps
                <em>text</em> o <em>text traduït</em> es retorna sense
                modificar. Si s'emplena el camp, el codi tindrà accés a les
                variables següents:
            </p>
            <dl>
                <dt><code>self</code></dt>
                <dd>
                    Una referència a l'objecte <code>Variable</code>.
                </dd>
                <dt><code>text</code></dt>
                <dd>
                    El text que s'utilitzarà com a valor per la variable.
                    Inicialitzat al text que es retornaria normalment.
                </dd>
            </dl>
        </detalls>
        """,
    es = u"""
        <p>
            Bloque de código Python que permite producir el valor de la
            variable programáticamente.
        </p>
        <details>
            <summary>Detalles</summary>
            <p>
                Si se deja en blanco, el valor obtenido a partir de los
                campos <em>texto</em> o <em>texto traducido</em> se devolverá
                sin modificar. Si se rellena este campo, el código tendrá
                acceso a las variables siguientes:
            </p>
            <dl>
                <dt><code>self</code></dt>
                <dd>
                    Una referencia a este objeto <code>Variable</code>.
                </dd>
                <dt><code>text</code></dt>
                <dd>
                    El texto que se utilitzará como valor para la variable.
                    Inicializado con el texto que se devolvería normalmente.
                </dd>
            </dl>
        </detalls>
        """,
    en = u"""
        <p>
            Python code block that makes it possible to change the value for
            this variable programmatically.
        </p>
        <details>
            <summary>Details</summary>
            <p>
                If left blank, the variable's value is produced by evaluating
                the <em>text</em> and <em>translated text</em> fields. If given
                a value, the code block will have access to the following
                variables:
            </p>
            <dl>
                <dt><code>self</code></dt>
                <dd>
                    A reference to this <code>Variable</code> object.
                </dd>
                <dt><code>text</code></dt>
                <dd>
                    The text that will be produced by this variable.
                    Initialized to the text that would be produced if the
                    expression field was empty.
                </dd>
            </dl>
        </detalls>
        """
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.schema.exceptions import SchemaIntegrityError
from cocktail.translations import translations

schema.Member._special_copy_keys.add("_annotation")
_base_deep_copy = schema.Member.__deepcopy__

def _deep_copy(self, memo):
    copy = _base_deep_copy(self, memo)
    copy._annotation = self._annotation
    return copy

schema.Member.__deepcopy__ = _deep_copy

schema.Member._annotation = None

def _get_annotation(self):
    return self._annotation

schema.Member.annotation = property(_get_annotation)

def add_annotation(self, **kwargs):

    if self._annotation:
        return self._annotation

    if isinstance(self, Annotation):
        raise SchemaIntegrityError(
            "Can't annotate an annotation member"
        )

    if not self.name:
        raise SchemaIntegrityError(
            "Can't annotate an anonymous member"
        )

    if not self.schema:
        raise SchemaIntegrityError(
            "Can't annotate a member that doesn't belong to a schema"
        )

    annotation = Annotation(self.name + "_annotation", **kwargs)
    annotation._annotation_target = self
    self._annotation = annotation
    annotation.translated = self.translated
    self.schema.add_member(annotation)
    return annotation

schema.Member.add_annotation = add_annotation


class Annotation(schema.String):

    _annotation_target = None
    listed_by_default = False
    edit_control = "cocktail.html.TextArea"

    def __translate__(self, language, **kwargs):
        return translations(
            "woost.extensions.annotations.Annotation-instance",
            instance = self,
            language = language,
            **kwargs
        )

    _special_copy_keys = set(schema.String._special_copy_keys)
    _special_copy_keys.add("_annotation_target")

    def __deepcopy__(self, memo):
        copy = schema.String.__deepcopy__(self, memo)
        copy._annotation_target = self._annotation_target
        return copy

    @property
    def annotation_target(self):
        return self._annotation_target

    @property
    def after_member(self):
        return self._annotation_target.name

    @property
    def member_group(self):
        return self._annotation_target.member_group




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations

for cls_name in ("Configuration", "Website"):
    translations.define(cls_name + ".external_files_host",
        ca = u"Domini fitxers externs",
        es = u"Dominio ficheros externos",
        en = u"External files host"
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from datetime import datetime
from cocktail.controllers import (
    Controller,
    FormProcessor,
    request_property,
    context
)
from woost import app
from woost.controllers.uploadform import UploadForm
from woost.controllers.formagreement import requires_agreement


class FormController(FormProcessor, Controller):

    is_transactional = True

    class UserForm(UploadForm):

        def __init__(self, *args, **kwargs):
            UploadForm.__init__(self, *args, **kwargs)
            for agreement in self.controller.block.agreements:
                agreement_member = requires_agreement(
                    self,
                    name = "agreement%d" % agreement.id,
                    document = agreement.document
                )

        @property
        def model(self):
            return self.controller.block.form_model

        @request_property
        def adapter(self):
            adapter = UploadForm.adapter(self)
            adapter.exclude("submit_date")
            return adapter

        def submit(self):
            UploadForm.submit(self)
            self.instance["submit_date"] = datetime.now()
            block = self.controller.block

            # User supplied initialization
            if block.submit_code:
                submit_context = {
                    "form": self,
                    "block": block
                }
                label = "%s #%s.submit_code" % (
                    block.__class__.__name__,
                    block.id
                )
                code = compile(block.submit_code, label, "exec")
                exec code in submit_context

            # Do something with the received data (by default, store it)
            self.handle_submitted_data()

        def handle_submitted_data(self):
            self.controller.block.submitted_data.append(self.instance)

        def after_submit(self):
            block = self.controller.block
            notification_receivers = block.notification_receivers
            email_messages = block.email_messages
            redirection = block.redirection
            redirection_parameters = {}

            # User supplied initialization
            if block.after_submit_code:
                after_submit_context = {
                    "form": self,
                    "block": block,
                    "notification_receivers": notification_receivers,
                    "email_messages": email_messages,
                    "redirection": redirection,
                    "redirection_parameters": redirection_parameters
                }
                label = "%s #%s.after_submit_code" % (
                    block.__class__.__name__,
                    block.id
                )
                code = compile(block.after_submit_code, label, "exec")
                exec code in after_submit_context
                notification_receivers = \
                    after_submit_context["notification_receivers"]
                email_messages = after_submit_context["email_messages"]
                redirection = after_submit_context["redirection"]
                redirection_parameters = \
                    after_submit_context["redirection_parameters"]

            # E-mail notifications
            for message in email_messages:
                message.send({
                    "block": block,
                    "form_data": self.instance,
                    "form_schema": self.model,
                    "notification_receivers": notification_receivers
                })

            # Redirection
            if redirection is None:
                publishable = app.publishable
                redirection = publishable.find_first_child_redirection_target()

            if redirection is not None:
                raise cherrypy.HTTPRedirect(redirection.get_uri(
                    parameters = redirection_parameters
                ))




# -*- coding: utf-8 -*-
u"""

.. moduleauthor:: Pepe Osca <pepe.osca@whads.com>
"""
import json
import urllib

import cherrypy
from cocktail.styled import styled
from cocktail.controllers import Controller, Location, session
from woost import app
from woost.controllers import CMSController
from .identityprovider import FacebookIdentityProvider

SESSION_PREFIX = "woost.extensions.identity.facebook."


class FacebookOAuthController(Controller):

    def resolve(self, path):

        if not path:
            raise cherrypy.HTTPError(400)

        provider_id = path[0]

        try:
            provider_id = int(provider_id)
        except ValueError:
            raise cherrypy.HTTPError(400)

        provider = FacebookIdentityProvider.get_instance(provider_id)
        if provider is None:
            raise cherrypy.NotFound()

        path.pop(0)
        return FacebookOAuthProviderController(provider)


class FacebookOAuthProviderController(Controller):

    def __init__(self, provider):
        Controller.__init__(self)
        self.provider = provider
        self.target_url = (
            session.get(SESSION_PREFIX + "target_url")
            or app.website.home.get_uri()
        )

    def step_url(self, step_number):
        location = Location.get_current_host()
        location.path_info = "/facebook_oauth/%d/step%d" % (
            self.provider.id,
            step_number
        )
        return unicode(location).decode("utf-8")

    @cherrypy.expose
    def step1(self, code = None, target_url = None, **kwargs):

        self.check_step1_errors(**kwargs)

        if target_url:
            self.target_url = target_url
            session[SESSION_PREFIX + "target_url"] = target_url

        if not code:
            params = {
                'client_id': self.provider.client_id,
                'redirect_uri': self.step_url(1),
                'scope': ','.join(self.provider.scope)
            }
            login_uri = 'https://www.facebook.com/dialog/oauth?' + \
                        urllib.urlencode(params)

            raise cherrypy.HTTPRedirect(login_uri)

        if self.provider.debug_mode:
            print styled("Facebook authorization code:", "magenta"), code

        params = {
            'client_id': self.provider.client_id,
            'redirect_uri': self.step_url(1),
            'client_secret': self.provider.client_secret,
            'code': code
        }
        token_uri = 'https://graph.facebook.com/v2.3/oauth/access_token?' \
                    + urllib.urlencode(params)

        json_file = urllib.urlopen(token_uri).readline()
        token_data = json.loads(json_file)

        if not token_data.get("access_token"):
            raise FacebookOAuthBadResponseException(
                token_data,
                "Expected an 'access_token' key"
            )

        session[SESSION_PREFIX + "credentials"] = token_data

        if self.provider.debug_mode:
            print styled("Facebook token data:", "magenta"),
            print token_data

        raise cherrypy.HTTPRedirect(self.step_url(2))

    @cherrypy.expose
    def step2(self):
        credentials = session.get(SESSION_PREFIX + "credentials")

        if not credentials:
            raise cherrypy.HTTPRedirect(self.step_url(1))

        fields = ['name', 'email']
        query = '{}{}{}{}'.format(
            'https://graph.facebook.com',
            '/me?',
            'fields=' + ','.join(fields),
            '&access_token=' + credentials['access_token']
        )

        user_data_file = urllib.urlopen(query).readline()
        user_data = json.loads(user_data_file)

        self.check_step2_errors(user_data)

        if self.provider.debug_mode:
            print styled("Facebook user profile:", "magenta"), user_data

        self.provider.login(user_data)
        del session[SESSION_PREFIX + "credentials"]

        raise cherrypy.HTTPRedirect(self.target_url)

    def check_step2_errors(self, result):

        if 'error' in result:
            if 'OAuthException' in result['error']:
                raise cherrypy.HTTPRedirect(self.step_url(1))
            else:
                msg = result.get('error_user_msg', 'Facebook user '
                                                   'authentification error')
                raise FacebookOAuthException(msg)

    def check_step1_errors(self, **kwargs):

        if "error" in kwargs:
            error_reason = kwargs.get('error_reason', 'Error')

            if error_reason == u'user_denied':
                raise cherrypy.HTTPRedirect(self.target_url)
            else:
                raise FacebookOAuthException(error_reason)


CMSController.facebook_oauth = FacebookOAuthController


class FacebookOAuthException(Exception):
    pass


class FacebookOAuthBadResponseException(FacebookOAuthException):

    def __init__(self, data, message):
        Exception.__init__(
            self,
            "Facebook returned %s; %s" % (json.dumps(data), message)
        )
        self.data = data
        self.message = message




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Block, Publishable
from woost.extensions.surveys.survey import Survey


class SurveyBlock(Block):

    type_group = "blocks.forms"
    view_class = "woost.extensions.surveys.SurveyView"
    default_controller = \
        "woost.extensions.surveys.surveycontroller.SurveyController"

    members_order = [
        "survey",
        "redirection"
    ]

    survey = schema.Reference(
        type = Survey,
        related_end = schema.Collection(),
        required = True,
        member_group = "content"
    )

    redirection = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.survey = self.survey




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.controllers.backoffice.useractions import UserAction
from .formblock import FormBlock


class ExportFormDataAction(UserAction):

    content_type = FormBlock
    included = frozenset(["item_buttons", "block_menu"])
    min = 1
    max = 1
    icon_uri = "/resources/images/export_xls.png"

    def get_url(self, controller, selection):
        return controller.contextual_uri(
            self.id,
            form = selection[0].id
        )


ExportFormDataAction("export_form_data").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from datetime import datetime, timedelta
from cocktail import schema
from cocktail.translations import get_language
from cocktail.caching import Cache
from cocktail.controllers import Location
from cocktail.persistence import datastore, PersistentMapping
from woost import app
from .item import Item


class CachingPolicy(Item):

    visible_from_root = False
    edit_form = "woost.views.CachingPolicyForm"

    groups_order = ["cache"]
    members_order = [
        "description",
        "important",
        "cache_enabled",
        "server_side_cache",
        "expiration_expression",
        "cache_tags_expression",
        "cache_key_expression",
        "condition"
    ]

    description = schema.String(
        descriptive = True,
        translated = True,
        spellcheck = True,
        listed_by_default = False
    )

    important = schema.Boolean(
        required = True,
        default = False
    )

    cache_enabled = schema.Boolean(
        required = True,
        default = True
    )

    server_side_cache = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False
    )

    expiration_expression = schema.CodeBlock(
        language = "python"
    )

    condition = schema.CodeBlock(
        language = "python"
    )

    cache_key_expression = schema.CodeBlock(
        language = "python"
    )

    cache_tags_expression = schema.CodeBlock(
        language = "python"
    )

    def applies_to(self, publishable, **context):
        expression = self.condition
        if expression:
            expression = expression.replace("\r", "")
            context["publishable"] = publishable
            exec expression in context
            return context.get("applies", False)

        return True

    def get_content_cache_key(self, publishable, **context):

        user = app.user

        cache_key = (
            str(Location.get_current(relative = False)),
            None
            if user is None or user.anonymous
            else tuple(role.id for role in user.roles)
        )
        key_qualifier = None
        expression = self.cache_key_expression

        if expression:
            expression = expression.replace("\r", "")
            context["publishable"] = publishable
            exec expression in context
            key_qualifier = context.get("cache_key")
        else:
            request = context.get("request")
            if request:
                key_qualifier = tuple(request.params.items())

        if key_qualifier:
            cache_key = cache_key + (key_qualifier,)

        return cache_key

    def get_content_expiration(self, publishable, base = None, **context):

        expression = self.expiration_expression
        expiration = base

        if expression:
            expression = expression.replace("\r", "")
            context["expiration"] = expiration
            context["publishable"] = publishable
            context["datetime"] = datetime
            context["timedelta"] = timedelta
            exec expression in context
            expiration = context.get("expiration")

        return expiration

    def get_content_tags(self, publishable, base = None, **context):

        tags = publishable.get_cache_tags(
            language = context.get("language") or get_language()
        )

        tags.add(self.main_cache_tag)

        if base:
            tags.update(base)

        expression = self.cache_tags_expression
        if expression:
            context["tags"] = tags
            exec expression in context
            tags = context.get("tags")

        return tags


# Utility functions
#------------------------------------------------------------------------------
def normalize_invalidation_date(value):

    if isinstance(value, Item):
        value = value.last_update_time

    elif hasattr(value, "__iter__"):
        max_date = None
        for item in value:
            date = normalize_invalidation_date(item)
            if date is None:
                continue
            if max_date is None or date > max_date:
                max_date = date

        value = max_date

    return value

def latest(selectable, *args, **kwargs):

    if not args and not kwargs \
    and hasattr(selectable, "get_last_instance_change"):
        return selectable.get_last_instance_change()

    return (
        selectable.select(
            order = Item.last_update_time.negative(),
            range = (0, 1)
        )
        .select(*args, **kwargs)
    )

def menu_items(publishable):
    items = []

    while publishable is not None:
        if hasattr(publishable, "children"):
            items.extend(publishable.children)
        if publishable.parent is None:
            items.append(publishable)
        publishable = publishable.parent

    return items

def file_date(publishable):
    try:
        ts = os.stat(publishable.file_path).st_mtime
    except IOError, OSError:
        return None

    return datetime.fromtimestamp(ts)

